# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestApi(TestApiG):
    environment = None

    def setUp(self):
        super(TestApi, self).setUp()
        self.create_gateway()
        self.gateway_id = TestApi.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"

        group_attrs = {
            "name": "api_group_001",
            "remark": "API group 1"
        }
        self.group = self.client.create_api_group(
            gateway=self.gateway_id,
            **group_attrs
        )
        self.assertIsNotNone(self.group.id)

        self.attrs = {
            "group_id": self.group.id,
            "name": "test_api_001",
            "auth_type": "IAM",
            "backend_type": "HTTP",
            "req_protocol": "HTTP",
            "req_uri": "/test/http",
            "remark": "Mock backend API",
            "type": 2,
            "req_method": "GET",
            "result_normal_sample": "Example success response",
            "result_failure_sample": "Example failure response",
            "tags": ["httpApi"],
            "backend_api": {
                "req_protocol": "HTTP",
                "req_method": "GET",
                "req_uri": "/test/benchmark",
                "timeout": 5000,
                "retry_count": "-1",
                "url_domain": "192.168.189.156:12346"
            },
        }
        self.api = self.client.create_api(
            gateway=self.gateway_id,
            **self.attrs
        )
        self.environment = self.client.create_environment(
            gateway=self.gateway_id,
            name="testPub",
            remark="test publish"
        )

        self.addCleanup(
            self.client.delete_api_group,
            gateway=self.gateway_id,
            api_group=self.group.id,
        )
        self.addCleanup(
            self.client.delete_api,
            gateway=self.gateway_id,
            api=self.api
        )
        self.addCleanup(
            self.client.delete_environment,
            gateway=self.gateway_id,
            environment=self.environment,
        )
        self.addCleanup(self.delete_gateway())

    def test_list_apis(self):
        a = list(self.client.apis(
            gateway=self.gateway_id))
        self.assertGreaterEqual(len(a), 1)

    def test_get_api(self):
        api = self.client.get_api(
            gateway=self.gateway_id,
            api=self.api.id)
        self.assertEqual(api.id, self.api.id)

    def test_update_api(self):
        self.attrs.update({
            "name": "test_api_001_updated",
            "type": 1,
            "req_method": "POST",
            "req_protocol": "HTTPS"
        })
        updated = self.client.update_api(
            gateway=self.gateway_id,
            api=self.api.id,
            **self.attrs
        )
        self.assertEqual(updated.name, self.attrs["name"])
        self.assertEqual(updated.type, self.attrs["type"])
        self.assertEqual(updated.req_method, self.attrs["req_method"])
        self.assertEqual(updated.req_protocol, self.attrs["req_protocol"])

    def test_online_offline_api(self):
        publish = self.client.publish_api(
            gateway=self.gateway_id,
            api=self.api.id,
            env=self.environment,
            remark="publish"
        )
        self.assertEqual(publish.api_name, self.api.name)
        self.assertEqual(publish.remark, "publish")

        offline = self.client.offline_api(
            gateway=self.gateway_id,
            api=self.api.id,
            env=self.environment,
            remark="offline"
        )
        self.assertEqual(offline.api_name, self.api.name)
        self.assertEqual(offline.remark, "offline")

    def test_check_api(self):
        attrs = {
            "type": "name",
            "name": self.api.name
        }
        check = self.client.check_api(
            gateway=self.gateway_id,
            **attrs
        )
        self.assertIsNotNone(check)

    def test_debug_api(self):
        attrs = {
            "mode": "DEVELOPER",
            "scheme": "HTTP",
            "method": "GET",
            "path": "/test/http"
        }
        debug = self.client.debug_api(
            gateway=self.gateway_id,
            api=self.api.id,
            **attrs
        )
        self.assertIsNotNone(debug)

    def test_online_offline_apis(self):
        publish = self.client.publish_apis(
            gateway=self.gateway_id,
            env=self.environment,
            remark="publish",
            apis=[self.api.id]
        )
        self.assertIsNotNone(publish)
        self.assertEqual(publish.success[0]["api_id"], self.api.id)

        offline = self.client.offline_apis(
            gateway=self.gateway_id,
            env=self.environment,
            remark="offline",
            apis=[self.api.id]
        )
        self.assertIsNotNone(offline)
        self.assertEqual(offline.success[0]["api_id"], self.api.id)

    def test_list_api_versions(self):
        versions = list(self.client.api_versions(
            gateway=self.gateway_id,
            api=self.api.id))
        self.assertIsNotNone(versions)

    def test_switch_api_versions(self):
        publish = self.client.publish_api(
            gateway=self.gateway_id,
            api=self.api.id,
            env=self.environment,
            remark="publish"
        )
        self.assertIsNotNone(publish)

        switch = self.client.switch_version(
            gateway=self.gateway_id,
            api=self.api.id,
            version_id=publish.version_id
        )
        self.assertIsNotNone(switch)

        offline = self.client.offline_api(
            gateway=self.gateway_id,
            env=self.environment,
            remark="offline",
            api=self.api.id,
        )
        self.assertIsNotNone(offline)

    # def test_list_runtime_definitions(self):
    #     definitions = list(self.client.api_runtime_definitions(
    #         gateway=self.gateway_id,
    #         api=self.api.id,
    #         env_id=self.environment.id
    #     ))
    #     self.assertIsNotNone(definitions)

    def test_list_api_version_details(self):
        publish = self.client.publish_api(
            gateway=self.gateway_id,
            api=self.api.id,
            env=self.environment,
            remark="publish"
        )
        self.assertIsNotNone(publish)

        details = list(self.client.api_version_details(
            gateway=self.gateway_id,
            version_id=publish.version_id
        ))
        self.assertIsNotNone(details)

        offline_version = self.client.take_api_version_offline(
            gateway=self.gateway_id,
            version_id=publish.version_id
        )
        self.assertIsNotNone(offline_version)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class MaintenanceTimeWindow(resource.Resource):

    resources_key = 'maintain_windows'

    base_path = '/instances/maintain-windows'

    # capabilities
    allow_list = True

    #: Properties
    #: An indicator of whether the maintenance time window is set to the
    # default time segment.
    default = resource.Body('default', type=bool)
    #: Sequence number of the maintenance time window.
    seq = resource.Body('seq', type=int)
    #: Start time of the maintenance time window.
    begin = resource.Body('begin')
    #: End time of the maintenance time window.
    end = resource.Body('end')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dcs.v1 import backup

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
FAKE_INSTANCE_ID = "some_fake_id"
EXAMPLE = {
    "backup_id": FAKE_ID,
    "status": "succeed",
    "remark": "remark",
    "period": "22:00-23:00",
    "progress": "100.00",
    "size": 126,
    "created_at": "2018-04-15T22:00:00.780Z",
    "updated_at": "2018-04-15T22:01:10.823Z",
    "backup_type": "auto",
    "backup_name": "backup_20180415230000",
    "error_code": None,
    "is_support_restore": "TRUE"
}


class TestBackup(base.TestCase):

    def test_basic(self):
        sot = backup.Backup()

        self.assertEqual('/instances/%(instance_id)s/backups', sot.base_path)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):

        sot = backup.Backup(instance_id=FAKE_INSTANCE_ID, **EXAMPLE)
        self.assertEqual(FAKE_INSTANCE_ID, sot.instance_id)
        self.assertEqual(EXAMPLE['backup_id'], sot.id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['remark'], sot.description)
        self.assertEqual(EXAMPLE['period'], sot.period)
        self.assertEqual(EXAMPLE['progress'], sot.progress)
        self.assertEqual(EXAMPLE['size'], sot.size)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['backup_type'], sot.type)
        self.assertEqual(EXAMPLE['backup_name'], sot.name)
        self.assertEqual(EXAMPLE['error_code'], sot.error_code)
        self.assertEqual(True, sot.is_restorable)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""NAT Gateway v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListNatGateways(command.Lister):

    _description = _("List Nat Gateway.")
    columns = ('Id', 'Name', 'Spec', 'Router Id', 'Status')

    def get_parser(self, prog_name):
        parser = super(ListNatGateways, self).get_parser(prog_name)

        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_("Specifies the ID of the NAT Gateway."),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Limit to fetch number of records."),
        )
        parser.add_argument(
            '--project-id',
            metavar='<project_id>',
            help=_("Specifies the project ID."),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the Name of the NAT Gateway."),
        )
        parser.add_argument(
            '--spec',
            metavar='<spec>',
            help=_("Specifies the type of the NAT Gateway. "
                   "The value of spec can be:\n"
                   "1: small type, which supports up to 10,000 "
                   "SNAT connections.\n"
                   "2: medium type, which supports up to 50,000 "
                   "SNAT connections.\n"
                   "3: large type, which supports up to 200,000 "
                   "SNAT connections.\n"
                   "4: extra-large type, which supports up to "
                   "1,000,000 SNAT connections."),
        )
        parser.add_argument(
            '--router-id',
            metavar='<router_id>',
            help=_("Specifies the router ID."),
        )
        parser.add_argument(
            '--internal-network-id',
            metavar='<internal_network_id>',
            help=_("Specifies the network ID of the downstream "
                   "interface (the next hop of the DVR) of the "
                   "NAT Gateway."),
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_("Specifies the status of the NAT Gateway.\n"
                   "ACTIVE: The resource status is normal.\n"
                   "PENDING_CREATE: The resource is being created.\n"
                   "PENDING_UPDATE: The resource is being updated.\n"
                   "PENDING_DELETE: The resource is being deleted.\n"
                   "EIP_FREEZED: The EIP of the resource is frozen.\n"
                   "INACTIVE: The resource status is abnormal."),
        )
        parser.add_argument(
            '--admin-state-up',
            metavar='<admin_state_up>',
            help=_("Specifies whether the NAT Gateway is enabled "
                   "or disabled. The value can be:\n"
                   "true: The NAT gateway is up.\n"
                   "false: The NAT gateway is down."),
        )
        parser.add_argument(
            '--created-at',
            metavar='<created_at>',
            help=_("Specifies when the NAT Gateway is created (UTC time). "
                   "Its valuerounds to 6 decimal places forseconds. "
                   "The format is yyyy-mm-ddhh:mm:ss."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat
        args_list = [
            'id',
            'limit',
            'project_id',
            'name',
            'spec',
            'router_id',
            'internal_network_id',
            'status',
            'admin_state_up',
            'created_at']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.gateways(**attrs)

        return (self.columns, (utils.get_item_properties(s, self.columns)
                               for s in data))


class ShowNatGateway(command.ShowOne):
    _description = _("Show NAT Gateway details")

    def get_parser(self, prog_name):
        parser = super(ShowNatGateway, self).get_parser(prog_name)
        parser.add_argument(
            'gateway',
            metavar='<gateway>',
            help=_("Specifies the Name or ID of the NAT Gateway."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat
        obj = client.find_gateway(parsed_args.gateway)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class CreateNatGateway(command.ShowOne):
    _description = _("Create new NAT Gateway")

    def get_parser(self, prog_name):
        parser = super(CreateNatGateway, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Specifies the name of the NAT Gateway."),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about "
                   "the NAT Gateway."),
        )
        parser.add_argument(
            '--spec',
            metavar='<spec>',
            required=True,
            help=_(
                "Specifies the type of the NAT Gateway. "
                "The value can be:\n"
                "1: small type, which supports up to 10,000 "
                "SNAT connections.\n"
                "2: medium type, which supports up to 50,000 "
                "SNAT connections.\n"
                "3: large type, which supports up to 200,000 "
                "SNAT connections.\n"
                "4: extra-large type, which supports up to "
                "1,000,000 SNAT connections."),
        )
        parser.add_argument(
            '--router-id',
            metavar='<router_id>',
            required=True,
            help=_("Specifies the VPC ID."),
        )
        parser.add_argument(
            '--internal-network-id',
            metavar='<internal_network_id>',
            required=True,
            help=_("Specifies the network ID of the downstream interface "
                   "(the next hop of the DVR) of the NAT Gateway."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat

        args_list = [
            'name',
            'description',
            'spec',
            'router_id',
            'internal_network_id']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.create_gateway(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UpdateNatGateway(command.ShowOne):
    _description = _("Update a NAT Gateway.")

    def get_parser(self, prog_name):
        parser = super(UpdateNatGateway, self).get_parser(prog_name)
        parser.add_argument(
            'gateway',
            metavar='<gateway>',
            help=_("Specifies the Name or ID of the NAT Gateway."),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the name of the NAT Gateway."),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary informationabout the NAT gateway."),
        )
        parser.add_argument(
            '--spec',
            metavar='<spec>',
            help=_("Specifies the type of the NAT Gateway."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat
        args_list = [
            'name', 'description', 'spec'
        ]
        attrs = {}
        for arg in args_list:
            if getattr(parsed_args, arg):
                attrs[arg] = getattr(parsed_args, arg)
        nat_gateway = client.find_gateway(parsed_args.gateway)

        obj = client.update_gateway(nat_gateway.id, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteNatGateway(command.Command):

    _description = _("Deletes NAT Gateway.")

    def get_parser(self, prog_name):
        parser = super(DeleteNatGateway, self).get_parser(prog_name)
        parser.add_argument(
            'gateway',
            metavar='<gateway>',
            nargs='+',
            help=_("Nat Gateway(s) to delete (Name or ID)"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat
        result = 0
        for gateway in parsed_args.gateway:
            try:
                obj = client.find_gateway(gateway)
                client.delete_gateway(obj.id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete Nat Gateway with "
                          "name or ID '%(gateway)s': %(e)s"),
                          {'gateway': gateway, 'e': e})
        if result > 0:
            total = len(parsed_args.gateway)
            msg = (_("%(result)s of %(total)s NAT Gateway(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json

from keystoneauth1 import _utils as utils
from keystoneauth1 import access
from keystoneauth1 import exceptions
from keystoneauth1 import loading
from keystoneauth1.identity.v3 import base


_logger = utils.get_logger(__name__)


class AssumeRoleMethod(base.AuthMethod):
    _method_parameters = ['user_id',
                          'username',
                          'user_domain_id',
                          'user_domain_name',
                          'password',
                          'roles',
                          'target_agency_name',
                          'target_domain_id',
                          'target_domain_name',
                          'target_project_id',
                          'target_project_name']

    def get_auth_data(self, session, auth, headers, **kwargs):
        user = {'password': self.password}

        if self.user_id:
            user['id'] = self.user_id
        elif self.username:
            user['name'] = self.username

            if self.user_domain_id:
                user['domain'] = {'id': self.user_domain_id}
            elif self.user_domain_name:
                user['domain'] = {'name': self.user_domain_name}

        return 'password', {'user': user}

    def get_assume_role_auth_data(self, session, auth, headers, **kwargs):
        agency = {'xrole_name': self.target_agency_name}

        if self.target_domain_id:
            agency['domain_id'] = self.target_domain_id
        elif self.target_domain_name:
            agency['domain_name'] = self.target_domain_name
        if self.roles:
            agency['restrict'] = {'roles': list(self.roles)}

        return 'assume_role', agency

    def get_cache_id_elements(self):
        return dict(('assume_role_%s' % p, getattr(self, p))
                    for p in self._method_parameters)


class Agency(base.AuthConstructor):
    """A plugin for authenticating with a username and password.
    It then does assume_role
    """
    _auth_method_class = AssumeRoleMethod

    def __init__(self, auth_url,
                 *args,
                 **kwargs):
        super(Agency, self).__init__(auth_url=auth_url,
                                     *args,
                                     **kwargs)
        self.target_project_id = kwargs.get('target_project_id')
        self.target_project_name = kwargs.get('target_project_name')
        self.target_domain_id = kwargs.get('target_domain_id')
        self.target_domain_name = kwargs.get('target_domain_name')

    def get_auth_ref(self, session, **kwargs):
        # First do regular authorization
        auth_access = super(Agency, self).get_auth_ref(session, **kwargs)
        # And now reauth with another scope
        headers = {
            'Accept': 'application/json',
            'X-Auth-Token': auth_access._auth_token
        }
        body = {'auth': {'identity': {}}}
        ident = body['auth']['identity']
        rkwargs = {}

        for method in self.auth_methods:
            name, auth_data = method.get_assume_role_auth_data(
                session, self, headers, request_kwargs=rkwargs)
            # NOTE(adriant): Methods like ReceiptMethod don't
            # want anything added to the request data, so they
            # explicitly return None, which we check for.
            if name:
                ident.setdefault('methods', []).append(name)
                ident[name] = auth_data

        if not ident:
            raise exceptions.AuthorizationFailure(
                'Authentication method required (e.g. password)')

        if self.target_project_id:
            body['auth']['scope'] = {'project': {'id': self.target_project_id}}
        elif self.target_project_name:
            scope = body['auth']['scope'] = {'project': {}}
            scope['project']['name'] = self.target_project_name
        # If project is not set - get a domain scope
        elif self.target_domain_id:
            body['auth']['scope'] = {'domain':
                                     {'id': self.target_domain_id}}
        elif self.target_domain_name:
            body['auth']['scope'] = {'domain':
                                     {'name': self.target_domain_name}}

        token_url = self.token_url

        if not self.auth_url.rstrip('/').endswith('v3'):
            token_url = '%s/v3/auth/tokens' % self.auth_url.rstrip('/')

        if not self.include_catalog:
            token_url += '?nocatalog'

        _logger.debug('Making authentication request to %s', token_url)
        resp = session.post(token_url, json=body, headers=headers,
                            authenticated=False, log=False, **rkwargs)

        try:
            _logger.debug(json.dumps(resp.json()))
            resp_data = resp.json()
        except ValueError:
            raise exceptions.InvalidResponse(response=resp)

        if 'token' not in resp_data:
            raise exceptions.InvalidResponse(response=resp)

        return access.AccessInfoV3(auth_token=resp.headers['X-Subject-Token'],
                                   body=resp_data)


class AgencyLoader(loading.BaseV3Loader):
    @property
    def plugin_class(self):
        return Agency

    def get_options(self, **kwargs):
        options = super(AgencyLoader, self).get_options()
        options.extend([
            loading.Opt('user-id', help='User ID'),
            loading.Opt('username', help='Username',
                        deprecated=[loading.Opt('user-name')]),
            loading.Opt('user-domain-id', help="User's domain id"),
            loading.Opt('user-domain-name', help="User's domain name"),
            loading.Opt('password', secret=True, prompt='Password: ',
                        help="User's password"),

            loading.Opt('target-agency-name', help="Agency name"),
            loading.Opt('target-project-id',
                        help="Project id available through agency"),
            loading.Opt('target-project-name',
                        help="Project name available through agency"),
            loading.Opt('target-domain-id',
                        help="Domain id available through agency"),
            loading.Opt('target-domain-name',
                        help="Domain name available through agency"),
            loading.Opt('roles',
                        help="List of the roles to request from agency"),
        ])
        return options

    def load_from_options(self, **kwargs):
        if (
            kwargs.get('username')
            and not (kwargs.get('user_domain_name')
                     or kwargs.get('user_domain_id'))
        ):
            m = "You have provided a username. In the V3 identity API a " \
                "username is only unique within a domain so you must " \
                "also provide either a user_domain_id or user_domain_name."
            raise exceptions.OptionError(m)
        if (
            not kwargs.get('target_agency_name')
            or not (kwargs.get('target_domain_id')
                    or kwargs.get('target_domain_name'))
        ):
            m = "Using agency based authorization requires " \
                "target_agency_name, target_domain_id or "\
                "target_domain_name at the very minimum"
            raise exceptions.OptionError(m)

        return super(AgencyLoader, self).load_from_options(**kwargs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.cbr.v3 import backup
from otcextensions.sdk.cbr.v3 import backup as backupSDK
from otcextensions.tests.unit.osclient.cbr.v3 import fakes


class TestBackup(fakes.TestCBR):

    def setUp(self):
        super(TestBackup, self).setUp()

    def test_add_children_to_backup_obj(self):
        obj = fakes.FakeBackup.create_one()

        column = ()
        data = ()
        verify_column = (
            'child_backup_1',
            'child_backup_2'
        )
        verify_data = (
            'child_backup_uuid_1',
            'child_backup_uuid_2'
        )

        data, column = backup._add_children_to_backup_obj(obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_flatten(self):
        obj = fakes.FakeBackup.create_one()

        flat_data = backup._flatten_backup(obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['checkpoint_id'],
            flat_data['created_at'],
            flat_data['description'],
            flat_data['expired_at'],
            flat_data['image_type'],
            flat_data['parent_id'],
            flat_data['project_id'],
            flat_data['protected_at'],
            flat_data['resource_az'],
            flat_data['resource_id'],
            flat_data['resource_name'],
            flat_data['resource_size'],
            flat_data['resource_type'],
            flat_data['status'],
            flat_data['updated_at'],
            flat_data['vault_id'],
            flat_data['provider_id']
        )

        cmp_data = (
            obj.id,
            obj.name,
            obj.checkpoint_id,
            obj.created_at,
            obj.description,
            obj.expired_at,
            obj.image_type,
            obj.parent_id,
            obj.project_id,
            obj.protected_at,
            obj.resource_az,
            obj.resource_id,
            obj.resource_name,
            obj.resource_size,
            obj.resource_type,
            obj.status,
            obj.updated_at,
            obj.vault_id,
            obj.provider_id
        )

        self.assertEqual(data, cmp_data)


class TestListBackup(fakes.TestCBR):

    objects = fakes.FakeBackup.create_multiple(3)

    columns = (
        'id',
        'name',
        'checkpoint_id',
        'created_at',
        'description',
        'expired_at',
        'image_type',
        'parent_id',
        'project_id',
        'protected_at',
        'resource_az',
        'resource_id',
        'resource_name',
        'resource_size',
        'resource_type',
        'status',
        'updated_at',
        'vault_id',
        'provider_id',
    )

    data = []

    for s in objects:
        flat_data = backup._flatten_backup(s)
        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['checkpoint_id'],
            flat_data['created_at'],
            flat_data['description'],
            flat_data['expired_at'],
            flat_data['image_type'],
            flat_data['parent_id'],
            flat_data['project_id'],
            flat_data['protected_at'],
            flat_data['resource_az'],
            flat_data['resource_id'],
            flat_data['resource_name'],
            flat_data['resource_size'],
            flat_data['resource_type'],
            flat_data['status'],
            flat_data['updated_at'],
            flat_data['vault_id'],
            flat_data['provider_id']
        ))

    def setUp(self):
        super(TestListBackup, self).setUp()

        self.cmd = backup.ListBackups(self.app, None)

        self.client.backups = mock.Mock()
        self.client.api_mock = self.client.backups

    def test_default(self):
        arglist = [
            '--checkpoint-id', 'checkpoint_uuid',
            '--dec', 'True',
            '--end-time', '2018-02-01T12:00:00Z',
            '--image-type', 'backup',
            '--limit', '5',
            '--marker', 'marker',
            '--member-status', 'accept',
            '--name', 'name',
            '--offset', '5',
            '--own-type', 'all_granted',
            '--parent-id', 'parent_uuid',
            '--resource-az', 'resource_az',
            '--resource-id', 'resource_uuid',
            '--resource-name', 'resource_name',
            '--resource-type', 'OS::Cinder::Volume',
            '--sort', 'sort',
            '--start-time', 'start_time',
            '--status', 'available',
            '--used-percent', 'used_percent',
            '--vault-id', 'vault_uuid'
        ]

        verifylist = [
            ('checkpoint_id', 'checkpoint_uuid'),
            ('dec', True),
            ('end_time', '2018-02-01T12:00:00Z'),
            ('image_type', 'backup'),
            ('limit', 5),
            ('marker', 'marker'),
            ('member_status', 'accept'),
            ('name', 'name'),
            ('offset', 5),
            ('own_type', 'all_granted'),
            ('parent_id', 'parent_uuid'),
            ('resource_az', 'resource_az'),
            ('resource_id', 'resource_uuid'),
            ('resource_name', 'resource_name'),
            ('resource_type', 'OS::Cinder::Volume'),
            ('sort', 'sort'),
            ('start_time', 'start_time'),
            ('status', 'available'),
            ('used_percent', 'used_percent'),
            ('vault_id', 'vault_uuid'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            checkpoint_id='checkpoint_uuid',
            dec=True,
            end_time='2018-02-01T12:00:00Z',
            image_type='backup',
            limit=5,
            marker='marker',
            member_status='accept',
            name='name',
            offset=5,
            own_type='all_granted',
            parent_id='parent_uuid',
            resource_az='resource_az',
            resource_id='resource_uuid',
            resource_name='resource_name',
            resource_type='OS::Cinder::Volume',
            sort='sort',
            start_time='start_time',
            status='available',
            used_percent='used_percent',
            vault_id='vault_uuid',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowBackup(fakes.TestCBR):

    object = fakes.FakeBackup.create_one()

    columns = (
        'id',
        'name',
        'checkpoint_id',
        'created_at',
        'description',
        'expired_at',
        'image_type',
        'parent_id',
        'project_id',
        'protected_at',
        'resource_az',
        'resource_id',
        'resource_name',
        'resource_size',
        'resource_type',
        'status',
        'updated_at',
        'vault_id',
        'provider_id'
    )

    flat_data = backup._flatten_backup(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['checkpoint_id'],
        flat_data['created_at'],
        flat_data['description'],
        flat_data['expired_at'],
        flat_data['image_type'],
        flat_data['parent_id'],
        flat_data['project_id'],
        flat_data['protected_at'],
        flat_data['resource_az'],
        flat_data['resource_id'],
        flat_data['resource_name'],
        flat_data['resource_size'],
        flat_data['resource_type'],
        flat_data['status'],
        flat_data['updated_at'],
        flat_data['vault_id'],
        flat_data['provider_id']
    )

    def setUp(self):
        super(TestShowBackup, self).setUp()

        self.cmd = backup.ShowBackup(self.app, None)

        self.client.find_backup = mock.Mock()

    def test_default(self):
        arglist = [
            'backup'
        ]
        verifylist = [
            ('backup', 'backup')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_backup.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_backup.assert_called_once_with(
            name_or_id='backup',
            ignore_missing=False,
        )

        self.data, self.columns = backup._add_children_to_backup_obj(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteBackup(fakes.TestCBR):

    def setUp(self):
        super(TestDeleteBackup, self).setUp()

        self.cmd = backup.DeleteBackup(self.app, None)

        self.client.delete_backup = mock.Mock()

    def test_delete(self):
        arglist = [
            'b1'
        ]
        verifylist = [
            ('backup', 'b1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_backup.side_effect = [{}]

        # Set the response for find_policy
        self.client.find_backup.side_effect = [
            backupSDK.Backup(id='b1')
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(
                backup='b1',
                ignore_missing=False),
        ]

        find_calls = [
            mock.call(
                name_or_id='b1',
                ignore_missing=False),
        ]

        self.client.delete_backup.assert_has_calls(delete_calls)
        self.client.find_backup.assert_has_calls(find_calls)
        self.assertEqual(1, self.client.delete_backup.call_count)


class TestRestoreBackup(fakes.TestCBR):

    object = fakes.FakeBackup.create_one()

    columns = (
        'id',
        'name',
        'checkpoint_id',
        'created_at',
        'description',
        'expired_at',
        'image_type',
        'parent_id',
        'project_id',
        'protected_at',
        'resource_az',
        'resource_id',
        'resource_name',
        'resource_size',
        'resource_type',
        'status',
        'updated_at',
        'vault_id',
        'provider_id',
    )

    flat_data = backup._flatten_backup(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['checkpoint_id'],
        flat_data['created_at'],
        flat_data['description'],
        flat_data['expired_at'],
        flat_data['image_type'],
        flat_data['parent_id'],
        flat_data['project_id'],
        flat_data['protected_at'],
        flat_data['resource_az'],
        flat_data['resource_id'],
        flat_data['resource_name'],
        flat_data['resource_size'],
        flat_data['resource_type'],
        flat_data['status'],
        flat_data['updated_at'],
        flat_data['vault_id'],
        flat_data['provider_id']
    )

    def setUp(self):
        super(TestRestoreBackup, self).setUp()

        self.cmd = backup.RestoreBackup(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.restore_data = mock.Mock()
        self.client.find_backup = mock.Mock()

    def test_default(self):
        arglist = [
            'backup_uuid',
            '--mappings', 'backup_id=backup_uuid volume_id=volume_uuid',
            '--server-id', 'server_uuid',
            '--volume-id', 'volume_uuid'
        ]
        verifylist = [
            ('backup', 'backup_uuid'),
            ('power_on', False),
            ('server_id', 'server_uuid'),
            ('volume_id', 'volume_uuid'),
            ('mappings', ['backup_id=backup_uuid volume_id=volume_uuid'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        #
        # Set the response
        self.client.restore_data.side_effect = [
            {}
        ]

        self.client.find_backup.side_effect = [
            backupSDK.Backup(id='backup_uuid')
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.restore_data.assert_called_once_with(
            backup='backup_uuid',
            mappings=[{'backup_id': 'backup_uuid',
                       'volume_id': 'volume_uuid'}],
            power_on=False,
            server_id='server_uuid',
            volume_id='volume_uuid'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class ParameterSpec(resource.Resource):
    #: Parameter name.
    label = resource.Body("label")
    #: Parameter value.
    value = resource.Body("value")
    #: Whethere parameter is required.
    required = resource.Body("required", type=bool)
    #: Placeholder.
    placeholder = resource.Body("placeholder")
    #: Tip.
    tip = resource.Body("tip", type=dict)


class HostPathSpec(resource.Resource):
    #: Local path of a training job.
    dest_path = resource.Body("dest_path")
    #: Whether dest_path is read-only.
    read_only = resource.Body("read_only", type=bool)
    #: Local path of a host.
    src_path = resource.Body("src_path")


class NfsSpec(resource.Resource):
    #: Local path of a training job.
    dest_path = resource.Body("dest_path")
    #: ID of an SFS Turbo file system.
    id = resource.Body("id")
    #: Whether dest_path is read-only.
    read_only = resource.Body("read_only", type=bool)
    #: Address of an SFS Turbo file system.
    src_path = resource.Body("src_path")


class VolumeSpec(resource.Resource):
    #: Storage volume of the host file system type.
    host_path = resource.Body("host_path", type=HostPathSpec)
    #: Storage volume of the shared file system type.
    nfs = resource.Body("nfs", type=NfsSpec)


class DataSourceSpec(resource.Resource):
    #: OBS bucket path.
    data_url = resource.Body("data_url")
    #: Dataset ID of a training job.
    dataset_id = resource.Body("dataset_id")
    #: Dataset version ID of a training job.
    dataset_version = resource.Body("dataset_version")
    #: Dataset type.
    type = resource.Body("type")


class SystemMetricListSpec(resource.Resource):
    #: CPU usage of a training job.
    cpu_usage = resource.Body("cpuUsage", type=list)
    #: GPU usage of a training job.
    gpu_util = resource.Body("gpuUtil", type=list)
    #: Memory usage of a training job.
    mem_usage = resource.Body("memUsage", type=list)
    #: Disk read rate of a training job.
    disk_read_rate = resource.Body("diskReadRate", type=list)
    #: Disk read rate of a training job.
    disk_write_rate = resource.Body("diskWriteRate", type=list)
    #: Received bytes rate of a training job.
    recv_bytes_rate = resource.Body("recvBytesRate", type=list)
    #: Send bytes rate of a training job.
    send_bytes_rate = resource.Body("sendBytesRate", type=list)
    #: GPU memory usage of a training job.
    gpu_mem_usage = resource.Body("gpuMemUsage", type=list)
    #: Interval
    interval = resource.Body("interval", type=int)


class ConfigSpec(resource.Resource):
    # Properties
    #: BuiltIn Algorithm Id.
    algorithm_id = resource.Body("algorithm_id")
    #: Training job additional Attributes.
    attributes = resource.Body("attributes", type=dict)
    #: Code directory of a training job, for example, /usr/app/.
    app_url = resource.Body("app_url")
    #: Billing resources.
    billing = resource.Body("billing", type=dict)
    #: Boot file of a training job, which needs to be stored
    #:  in the code directory.
    boot_file_url = resource.Body("boot_file_url")
    #: Number of cores of the resource specifications.
    core = resource.Body("core")
    #: CPU memory of the resource specifications.
    cpu = resource.Body("cpu")
    #: Timestamp when a training job is created.
    created_at = resource.Body("create_time", type=int)
    #: Whether a version is created when a training job is created.
    #:  Default value.
    create_version = resource.Body("create_version", type=bool)
    #: Dataset of a training job.
    data_source = resource.Body(
        "data_source", type=list, list_type=DataSourceSpec
    )
    #: OBS URL of the dataset required by a training job.
    data_url = resource.Body("data_url")
    #: Dataset ID of a training job.
    dataset_id = resource.Body("dataset_id")
    #: Dataset of a training job.
    dataset_name = resource.Body("dataset_name")
    #: Dataset version ID of a training job.
    dataset_version_id = resource.Body("dataset_version_id")
    #: Dataset of a training job.
    dataset_version_name = resource.Body("dataset_version_name")
    #: Description of a training job.
    description = resource.Body("description")
    #: Training job running duration, in milliseconds.
    duration = resource.Body("duration", type=int)
    #: ID of the engine selected for a training job.
    engine_id = resource.Body("engine_id", type=int)
    #: Name of the engine selected for a training job.
    engine_name = resource.Body("engine_name")
    #: Engine type of a training job.
    engine_type = resource.Body("engine_type", type=int)
    #: Version of the engine selected for a training job.
    engine_version = resource.Body("engine_version")
    #: Error code of a failed API call.
    error_code = resource.Body("error_code")
    #: Error message of a failed API call.
    error_message = resource.Body("error_message")
    #: Resource specification selected for a training job.
    flavor_code = resource.Body("flavor_code")
    #: Resource specifications details.
    flavor_info = resource.Body("flavor_info", type=dict)
    #: Resource specification type selected for a training job type.
    flavor_type = resource.Body("flavor_type")
    #: GPU memory unit.
    gpu_memory_unit = resource.Body("gpu_memory_unit")
    #: Number of GPUs of the resource specifications.
    gpu_num = resource.Body("gpu_num", type=int)
    #: GPU type of the resource specifications.
    gpu_type = resource.Body("gpu_type")
    #: Whether the resource specification used for training is free.
    is_free = resource.Body("is_free")
    #: Whether the request is successful.
    is_success = resource.Body("is_success", type=bool)
    #: Whether training job is in Zombie to state.
    is_zombie = resource.Body("is_zombie", type=bool)
    #: Description of a training job.
    job_description = resource.Body("job_desc")
    #: ID of a training job.
    job_id = resource.Body("job_id", type=int)
    #: Name of a training job.
    job_name = resource.Body("job_name")
    #: Training job type.
    job_type = resource.Body("job_type", type=int)
    #: OBS URL of the logs of a training job.
    log_url = resource.Body("log_url")
    #: Max count.
    max_num = resource.Body("max_num", type=int)
    #: Memory unit.
    memory_unit = resource.Body("memory_unit")
    #: Model metrics of a training job.
    model_metric_list = resource.Body("model_metric_list")
    #: ID of the built-in model of a training job.
    model_id = resource.Body("model_id", type=int)
    #: Name of a training job.
    name = resource.Body("name", alias="job_name")
    #: Local mount path of SFS Turbo (NAS).
    nas_mount_path = resource.Body("nas_mount_path")
    #: Shared path of SFS Turbo (NAS).
    nas_share_addr = resource.Body("nas_share_addr")
    #: Only NFS is supported.
    nas_type = resource.Body("nas_type")
    #: Whether no resource.
    no_resource = resource.Body("no_resource", type=bool)
    #: NPU Info.
    npu_info = resource.Body("npu_info")
    #: Running parameters of a training job.
    parameter = resource.Body("parameter", type=list, list_type=ParameterSpec)
    #: Pod Version.
    pod_version = resource.Body("pod_version")
    #: ID of a resource pool.
    pool_id = resource.Body("pool_id")
    #: Name of a resource pool.
    pool_name = resource.Body("pool_name")
    #: Resource pool type.
    pool_type = resource.Body("pool_type")
    #: Name of the previous version of a training job.
    pre_version_id = resource.Body("pre_version_id", type=int)
    #: Charged resource ID of a training job.
    resource_id = resource.Body("resource_id")
    #: Model Id used for re-training.
    retrain_model_id = resource.Body("retrain_model_id")
    #: Resource specifications selected for a training job.
    spec_code = resource.Body("spec_code")
    #: ID of the resource specifications selected for a training job.
    spec_id = resource.Body("spec_id", type=int)
    #: Training start time.
    started_at = resource.Body("start_time", type=int)
    #: Status of a training job.
    status = resource.Body("status", type=int)
    #: System monitoring metrics of a training job.
    system_metric_list = resource.Body(
        "system_metric_list", type=SystemMetricListSpec
    )
    #: OBS URL of the output file of a training job.
    train_url = resource.Body("train_url")
    #: Boot command used to start the container of a
    #:  custom image of a training job.
    user_command = resource.Body("user_command")
    #: SWR URL of a custom image used by a training job.
    user_image_url = resource.Body("user_image_url")
    #: Number of versions of a training job.
    version_count = resource.Body("version_count", type=int)
    #: Version ID of a training job.
    version_id = resource.Body("version_id", type=int)
    #: Version name of a training job.
    version_name = resource.Body("version_name")
    #: Storage volume that can be used by a training job.
    volumes = resource.Body("volumes", type=list, list_type=VolumeSpec)
    #: Number of workers in a training job.
    worker_server_num = resource.Body("worker_server_num", type=int)


class TrainingJob(ConfigSpec):
    base_path = "/training-jobs"

    resources_key = "jobs"

    # capabilities
    allow_commit = True
    allow_create = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        "limit",
        "order",
        "offset",
        "search_content",
        "sort_by",
        "status",
        "workspace_id",
        limit="per_page",
        offset="page",
        sort_by="sortBy",
    )

    # Properties
    #: Parameters for creating a training job.
    config = resource.Body("config", type=ConfigSpec)
    #: ID of a training job.
    job_id = resource.Body("job_id", type=int, alternate_id=True)
    #: Training job name.
    job_name = resource.Body("job_name")
    #: User Info.
    user = resource.Body("user", type=dict)
    #: Workspace where a job resides.
    workspace_id = resource.Body("workspace_id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.dis.v2 import app
from otcextensions.sdk.dis.v2 import app as sdk_app
from otcextensions.common import cli_utils
from otcextensions.tests.unit.osclient.dis.v2 import fakes

from openstackclient.tests.unit import utils as tests_utils


class TestListApps(fakes.TestDis):

    objects = fakes.FakeApp.create_multiple(3)

    column_list_headers = (
        'App Name',
        'Id',
        'Created At',
    )

    columns = (
        'name',
        'id',
        'created_at',
    )

    data = []

    for s in objects:
        data.append((
            s.name,
            s.id,
            cli_utils.UnixTimestampFormatter(s.created_at),
        ))

    def setUp(self):
        super(TestListApps, self).setUp()

        self.cmd = app.ListApps(self.app, None)

        self.client.apps = mock.Mock()
        self.client.api_mock = self.client.apps

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--limit', '1',
            '--start-app-name', '2',
            '--stream-name', '3',
        ]

        verifylist = [
            ('limit', 1),
            ('start_app_name', '2'),
            ('stream_name', '3'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            start_app_name='2',
            stream_name='3',
        )


class TestCreateApp(fakes.TestDis):

    columns = (
        'name',
    )

    def setUp(self):
        super(TestCreateApp, self).setUp()

        self.cmd = app.CreateApp(self.app, None)

    def test_create(self):
        arglist = [
            'test-app',
        ]
        verifylist = [
            ('appName', 'test-app'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        attrs = {}
        attrs.update(
            app_name='test-app',
        )

        self.client.create_app = mock.Mock(
            return_value=sdk_app.App(**attrs))

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_app.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)


class TestShowApp(fakes.TestDis):

    _data = fakes.FakeApp.create_one()
    columns = (
        'commit_checkpoint_stream_names',
        'created_at',
        'id',
        'name',
    )

    data = fakes.gen_data(_data, columns, app._formatters)

    def setUp(self):
        super(TestShowApp, self).setUp()

        self.cmd = app.ShowApp(self.app, None)

        self.client.get_app = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(tests_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_show(self):
        arglist = [
            self._data.name,
        ]

        verifylist = [
            ('appName', self._data.name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_app.assert_called_with(self._data.name)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_dis_app',
        ]

        verifylist = [
            ('appName', 'unexist_dis_app'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        get_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_app = (
            mock.Mock(side_effect=get_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.get_app.assert_called_with('unexist_dis_app')


class TestDeleteApp(fakes.TestDis):

    _data = fakes.FakeApp.create_multiple(2)

    def setUp(self):
        super(TestDeleteApp, self).setUp()

        self.client.delete_app = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = app.DeleteApp(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].name,
        ]

        verifylist = [
            ('appName', [self._data[0].name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_app.assert_called_with(self._data[0].name)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for dis_app in self._data:
            arglist.append(dis_app.name)

        verifylist = [
            ('appName', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for dis_app in self._data:
            calls.append(call(dis_app.name))
        self.client.delete_app.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].name,
            'unexist_dis_app',
        ]
        verifylist = [
            ('appName', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # delete_mock_result = [None, exceptions.CommandError]

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 DIS App(s) failed to delete.', str(e))

        calls = [call(self._data[0].name), call('unexist_dis_app')]
        self.client.delete_app.assert_has_calls(calls)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource
from openstack import utils


STATUS_MAP = {'100': 'CREATING', '200': 'AVAILABLE', '303': 'UNAVAILABLE'}


class FailedReasonSpec(resource.Resource):
    #: Error code.
    error_code = resource.Body('errorCode')
    #: Error details.
    error_msg = resource.Body('errorMsg')


class TagSpec(resource.Resource):
    #: Tag key.
    key = resource.Body('key')
    #: Tag value.
    value = resource.Body('value')


class WhiteListSpec(resource.Resource):
    #: Whether the network access control is enabled.
    is_whitelist_enabled = resource.Body('enableWhiteList', type=bool)
    #: Whitelist for public network access.
    whitelist = resource.Body('whiteList')


class KibanaRespSpec(resource.Resource):
    #: Bandwidth range.
    eip_size = resource.Body('eipSize', type=int)
    #: Kibana public network access information.
    elb_whitelist_resp = resource.Body('elbWhiteListResp', type=WhiteListSpec)
    #: Specifies the IP address for accessing Kibana.
    public_kibana_ip = resource.Body('publicKibanaIp')


class NicsSpec(resource.Resource):
    #: Subnet ID.
    network_id = resource.Body('netId')
    #: Security group ID.
    security_group_id = resource.Body('securityGroupId')
    #: VPC ID, which is used for configuring cluster network.
    router_id = resource.Body('vpcId')


class VolumeSpec(resource.Resource):
    #: Volume size.
    size = resource.Body('size', type=int)
    #: Volume type.
    type = resource.Body('type')
    #: Volume type.
    volume_type = resource.Body('volume_type')


class NodeSpec(resource.Resource):
    #: AZ of a node.
    availability_zone = resource.Body('azCode')
    #: Instance ID.
    id = resource.Body('id')
    #: Instance IP address.
    ip = resource.Body('ip')
    #: Instance name.
    name = resource.Body('name')
    #: Node specifications.
    flavor = resource.Body('specCode')
    #: Node status value.
    status = resource.Body('status')
    #: Type of the current node.
    type = resource.Body('type')
    #: Instance disk information.
    volume = resource.Body('volume', type=VolumeSpec)


class InstanceSpec(resource.Resource):
    #: Availability zone (AZ).
    availability_zone = resource.Body('availability_zone')
    #: Instance flavor name.
    flavor = resource.Body('flavorRef')
    #: Subnet information.
    nics = resource.Body('nics', type=NicsSpec)
    #: Information about the volume.
    volume = resource.Body('volume', type=VolumeSpec)


class DiskEncryptionSpec(resource.Resource):
    #: Key ID.
    system_cmkid = resource.Body('systemCmkid')
    #: Value 1 indicates encryption is performed, and value 0 indicates
    #:  encryption is not performed.
    system_encrypted = resource.Body('systemEncrypted')


class DatastoreSpec(resource.Resource):
    #: Engine type.
    type = resource.Body('type')
    #: Whether security mode is supported.
    version = resource.Body('version')


class BackupStrategySpec(resource.Resource):
    #: IAM agency used to access OBS.
    agency = resource.Body('agency')
    #: Storage path of the snapshot in the OBS bucket.
    base_path = resource.Body('basePath')
    #: OBS bucket used for storing backup.
    bucket = resource.Body('bucket')
    #: Number of days for which automatically created snapshots are reserved.
    keepday = resource.Body('keepday', type=int)
    #: Time when a snapshot is created every day.
    period = resource.Body('period')
    #: Prefix of the name of the snapshot that is automatically created.
    prefix = resource.Body('prefix')


class Cluster(resource.Resource):
    base_path = '/clusters'

    resources_key = 'clusters'
    resource_key = 'cluster'

    # capabilities
    allow_create = True
    allow_list = True
    allow_delete = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters('start', 'limit')

    # Properties
    #: Cluster behavior progress, which shows the progress of cluster creation
    #:  and scaling in percentage.
    action_progress = resource.Body('actionProgress', type=dict)
    #: Current behavior of a cluster.
    actions = resource.Body('actions', type=list)
    #: Password of the cluster user admin in security mode.
    admin_pwd = resource.Body('adminPwd')
    #: Automatic snapshot creation.
    backup_strategy = resource.Body('backupStrategy', type=BackupStrategySpec)
    #: Public network bandwidth.
    bandwidth_size = resource.Body('bandwidthSize', type=int)
    #: KMS Key ID.
    cmk_id = resource.Body('cmk_id')
    #: Cluster creation time.
    created_at = resource.Body('created')
    #: Type of the data search engine.
    datastore = resource.Body('datastore', type=DatastoreSpec)
    #: Whether disks are encrypted.
    disk_encryption = resource.Body('diskEncryption', type=DiskEncryptionSpec)
    #: Public network access information.
    elb_whitelist = resource.Body('elbWhiteList', type=WhiteListSpec)
    #: IP address and port number of the user used to access the VPC.
    endpoints = resource.Body('endpoint', type=list)
    #: ID of the enterprise project that a cluster belongs to.
    enterprise_project_id = resource.Body('enterpriseProjectId')
    #: Error object
    error = resource.Body('failed_reasons', type=FailedReasonSpec)
    #: Public IP address information.
    floating_ip = resource.Body('publicIp')
    #: Cluster ID.
    id = resource.Body('id')
    #: Instance.
    instance = resource.Body('instance', type=InstanceSpec)
    #: Number of clusters.
    instance_num = resource.Body('instanceNum', type=int)
    #: Whether to enable authentication.
    is_authority_enabled = resource.Body('authorityEnable', type=bool)
    #: Whether cluster is billed.
    is_billed = resource.Body('period')
    #: Whether the snapshot function is enabled.
    is_backup_enabled = resource.Body('backupAvailable', type=bool)
    #: Whether disks are encrypted.
    is_disk_encrypted = resource.Body('diskEncrypted', type=bool)
    #: Communication encryption status.
    is_https_enabled = resource.Body('httpsEnable', type=bool)
    #: Network ID.
    network_id = resource.Body('subnetId')
    #: List of node objects.
    nodes = resource.Body('instances', type=list, list_type=NodeSpec)
    #: Cluster name.
    name = resource.Body('name')
    #: Kibana public network access information.
    public_kibana_resp = resource.Body('publicKibanaResp', type=KibanaRespSpec)
    #: Router (VPC) ID.
    router_id = resource.Body('vpcId')
    #: Security group ID.
    security_group_id = resource.Body('securityGroupId')
    #: Cluster status.
    status = resource.Body('status')
    #: Cluster status code.
    status_code = resource.Body('status_code', type=int)
    #: Cluster tags.
    tags = resource.Body('tags', type=list, list_type=TagSpec)
    #: Last modification time of a cluster.
    updated_at = resource.Body('updated')
    #: Endpoint IP address.
    vpcep_ip = resource.Body('vpcepIp')

    def _action(self, session, action, body=None):
        """Preform actions given the message body."""
        uri = utils.urljoin('clusters', self.id, action)
        response = session.post(uri, json=body)
        exceptions.raise_from_response(response)

    def restart(self, session):
        """Restart the cluster."""
        self._action(session, 'restart')

    def extend(self, session, add_nodes):
        """Scaling Out a Cluster with only Common Nodes."""
        if not 0 < add_nodes <= 32:
            raise exceptions.SDKException('CSS Cluster size can be [1..32]')
        self._action(session, 'extend', {'grow': {'modifySize': add_nodes}})

    def update_flavor(
        self, session, new_flavor, node_type=None, check_replica=True
    ):
        """Modify cluster specifications."""
        body = {
            'newFlavorId': new_flavor
        }
        if check_replica in (True, False):
            body['needCheckReplica'] = check_replica

        url_path = node_type + '/flavor' if node_type else 'flavor'

        self._action(session, url_path, body)

    def update_name(self, session, new_name):
        """Update the name of the CSS cluster."""
        body = {'displayName': new_name}
        self._action(session, 'changename', body)

    def update_password(self, session, new_password):
        """Update the password of the CSS cluster."""
        body = {'newpassword': new_password}
        self._action(session, 'password/reset', body)

    def update_security_mode(
        self,
        session,
        https_enable=None,
        authority_enable=None,
        admin_pwd=None,
    ):
        body = {}
        if https_enable in (True, False):
            body['httpsEnable'] = https_enable
        if authority_enable in (True, False):
            body['authorityEnable'] = authority_enable
        if admin_pwd:
            body['adminPwd'] = admin_pwd

        self._action(session, 'mode/change', body)

    def update_security_group(self, session, security_group_id):
        """Update security group of the CSS cluster."""
        body = {'security_group_ids': security_group_id}
        self._action(session, 'sg/change', body)

    def update_kernel(
        self,
        session,
        target_image_id,
        upgrade_type,
        indices_backup_check,
        agency,
        cluster_load_check=True,
    ):
        """Update the kernel of the CSS cluster."""
        body = {
            'target_image_id': target_image_id,
            'upgrade_type': upgrade_type,
            'indices_backup_check': indices_backup_check,
            'agency': agency,
        }
        if cluster_load_check in (True, False):
            body['cluster_load_check'] = cluster_load_check

        self._action(session, 'inst-type/all/image/upgrade', body)

    def scale_in(self, session, nodes):
        """Scale in a cluster by removing specified nodes."""
        if not isinstance(nodes, list):
            raise exceptions.SDKException(
                'nodes parameter should be list type'
            )

        body = {'shrinkNodes': nodes}

        self._action(session, 'node/offline', body)

    def scale_in_by_node_type(self, session, nodes):
        """Remove instances of specific types and reduce instance
        storage capacity in a cluster."""
        if not isinstance(nodes, list):
            raise exceptions.SDKException(
                'nodes parameter should be list of dictionary items'
            )

        data = []

        for item in nodes:
            node_type = item.get('type')
            reduced_node_num = item.get('reducedNodeNum')
            if node_type not in (
                'ess',
                'ess-master',
                'ess-client',
                'ess-cold',
            ):
                raise ValueError(
                    f'Unsupported node type: {node_type}. Supported '
                    'node_type can be ess, ess-master, ess-client, ess-cold,'
                )
            if not isinstance(reduced_node_num, int) or reduced_node_num < 1:
                raise ValueError(
                    f'Reduced node number for {node_type} must be '
                    'a non-negative integer.'
                )

            data.append(
                {
                    'type': node_type.replace('_', '-'),
                    'reducedNodeNum': reduced_node_num,
                }
            )
        endpoint = session.get_endpoint().replace("v1.0", "v1.0/extend")

        url = utils.urljoin(endpoint, 'clusters', self.id, 'role/shrink')
        response = session.post(url, json={'shrink': data})
        exceptions.raise_from_response(response)

    def replace_node(self, session, node_id):
        """Replacing cluster node."""
        uri = utils.urljoin(
            'clusters', self.id, 'instance', node_id, 'replace'
        )
        response = session.put(uri)
        exceptions.raise_from_response(response)

    def add_nodes(self, session, node_type, flavor, node_size, volume_type):
        """Adding master and client node to cluster."""

        body = {
            'type': {
                'flavor_ref': flavor,
                'node_size': node_size,
                'volume_type': volume_type,
            }
        }

        self._action(session, f'type/{node_type}/independent', body)

    def retry_upgrade_job(self, session, job_id, retry_mode=None):
        """Retry a task or terminate the impact of a task."""
        uri = utils.urljoin('clusters', self.id, 'upgrade', job_id, 'retry')

        params = {}

        if retry_mode:
            params['retry_mode'] = 'abort'

        response = session.put(uri, params=params)
        exceptions.raise_from_response(response)

    @classmethod
    def existing(cls, connection=None, **kwargs):
        """Create an instance of an existing remote resource.

        When creating the instance set the ``_synchronized`` parameter
        of :class:`Resource` to ``True`` to indicate that it represents the
        state of an existing server-side resource. As such, all attributes
        passed in ``**kwargs`` are considered "clean", such that an immediate
        :meth:`update` call would not generate a body of attributes to be
        modified on the server.

        :param dict kwargs: Each of the named arguments will be set as
            attributes on the resulting Resource object.
        """
        if 'status' in kwargs.keys():
            kwargs['status_code'] = kwargs['status']
            kwargs['status'] = STATUS_MAP.get(str(kwargs['status']), 'ERROR')
        if kwargs.get('endpoint'):
            kwargs['endpoint'] = kwargs['endpoint'].split(',')
        return cls(_synchronized=True, connection=connection, **kwargs)

    def _translate_response(
        self,
        response,
        has_body=None,
        error_message=None,
        *,
        resource_response_key=None,
    ):
        """Given a KSA response, inflate this instance with its data

        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body

        exceptions.raise_from_response(response, error_message=error_message)

        if has_body:
            try:
                body = response.json()
                if resource_response_key and resource_response_key in body:
                    body = body[resource_response_key]
                elif self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]

                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                body.pop('self', None)

                if 'status' in body.keys():
                    body['status_code'] = body['status']
                    body['status'] = STATUS_MAP.get(
                        str(body['status']), 'ERROR'
                    )
                if body.get('endpoint'):
                    body['endpoint'] = body['endpoint'].split(',')
                body_attrs = self._consume_body_attrs(body)
                if self._allow_unknown_attrs_in_body:
                    body_attrs.update(body)
                    self._unknown_attrs_in_body.update(body)
                elif self._store_unknown_attrs_as_properties:
                    body_attrs = self._pack_attrs_under_properties(
                        body_attrs, body
                    )

                self._body.attributes.update(body_attrs)
                self._body.clean()
                if self.commit_jsonpatch or self.allow_patch:
                    # We need the original body to compare against
                    self._original_body = body_attrs.copy()
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())


class ExtendClusterNodes(resource.Resource):
    base_path = '/clusters/%(cluster_id)s/role_extend'

    allow_create = True

    cluster_id = resource.URI('cluster_id')

    # Properties
    #: Detailed description about the cluster scale-out request.
    grow = resource.Body('grow', type=list, list_type=dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class HostInstanceCapacity(resource.Resource):
    #: Specifies flavor ID
    flavor = resource.Body('flavor')


class HostProperties(resource.Resource):
    #: Specifies the VM flavors placed on the DeH
    available_instance_capacities = \
        resource.Body('available_instance_capacities',
                      type=list,
                      list_type=HostInstanceCapacity)
    #: Specifies the number of host physical cores
    cores = resource.Body('cores', type=int)
    #: Specifes the DeH type
    host_type = resource.Body('host_type')
    #: Specifes the DeH name of type
    host_type_name = resource.Body('host_type_name')
    #: Specifies the size of host physical memory (MB)
    memory = resource.Body('memory', type=int)
    #: Specifies the number of host physical sockets
    sockets = resource.Body('sockets', type=int)
    #: Specifies the number of host vCPUs
    vcpus = resource.Body('vcpus', type=int)


class Host(resource.Resource):
    resource_key = 'dedicated_host'
    resources_key = 'dedicated_hosts'
    base_path = '/dedicated-hosts'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'id', 'name', 'host_type', 'host_type_name',
        'flavor', 'status', 'tenant', 'availability_zone',
        'changes_since', 'tags', 'instance_uuid', 'released_at',
        changes_since='changes-since')

    #: Properties
    #: Specifies DeH ID
    id = resource.Body('dedicated_host_id', alternate_id=True)
    #: Time at which the DeH has been allocated
    allocated_at = resource.Body('allocated_at')
    #: Specifies whether to allow a VM to be placed on this available
    #: host if its DeH ID is not specified during its creation
    #: value: ['on', 'off']
    auto_placement = resource.Body('auto_placement')
    #: Specifies the number of available vCPUs for the DeH
    available_vcpus = resource.Body('available_vcpus', type=int)
    #: Specifies the number of available memory for the DeH
    available_memory = resource.Body('available_memory', type=int)
    #: Specifies the AZ to which the DeH belongs
    availability_zone = resource.Body('availability_zone')
    #: list of created DeH hosts (during create)
    dedicated_host_ids = resource.Body('dedicated_host_ids', type=list)
    #: Specifies the property of host
    host_properties = resource.Body('host_properties', type=HostProperties)
    #: Specifies the DeH type (for creation)
    host_type = resource.Body('host_type')
    #: Specifies the number of the placed VMs
    instance_total = resource.Body('instance_total', type=int)
    #: Specifies the VMs started on the DeH.
    #: The "Querying DeHs" intercace does not display this parameter.
    instance_uuids = resource.Body('instance_uuids', type=list)
    #: Specifies the tenant who owns the DeH
    project_id = resource.Body('project_id')
    #: Specifies the number of allocated DeHs (during creation).
    quantity = resource.Body('quantity', type=int)
    #: Time at which the DeH has been released
    released_at = resource.Body('released_at')
    #: Specifies the DeH status.
    #: The value can be available, fault or released
    status = resource.Body('state')
    #: Tag.
    tags = resource.Body('tags', type=list)

    def fetch_tags(self, session):
        """Lists tags set on the entity.

        :param session: The session to use for making this request.
        :return: The list with tags attached to the entity
        """
        url = utils.urljoin('dedicated-host-tags', self.id, 'tags')
        session = self._get_session(session)
        response = session.get(url)
        exceptions.raise_from_response(response)
        # NOTE(gtema): since this is a common method
        # we can't rely on the resource_key, because tags are returned
        # without resource_key. Do parse response here
        json = response.json()
        if 'tags' in json:
            self._body.attributes.update({'tags': json['tags']})
        return self

    def add_tags(self, session, tags=[]):
        """Sets/Replaces all tags on the resource.

        :param session: The session to use for making this request.
        :param list tags: List with tags to be set on the resource
        """
        url = utils.urljoin('dedicated-host-tags', self.id, 'tags', 'action')
        session = self._get_session(session)
        response = session.post(url, json={'action': 'create', 'tags': tags})
        exceptions.raise_from_response(response)
        self.fetch_tags(session)
        return self

    def remove_tags(self, session, tags=[]):
        """Sets/Replaces all tags on the resource.

        :param session: The session to use for making this request.
        :param list tags: List with tags to be removed on the resource
        """
        url = utils.urljoin('dedicated-host-tags', self.id, 'tags', 'action')
        session = self._get_session(session)
        response = session.post(url, json={'action': 'delete', 'tags': tags})
        exceptions.raise_from_response(response)
        self.fetch_tags(session)
        return self

    # Create method for host does not accept resource key while creation
    def create(self, session, prepend_key=False, base_path=None, **params):
        return super(Host, self).create(
            session,
            prepend_key=prepend_key,
            base_path=base_path,
            **params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.vlb import TestVlb


class TestSecurityPolicy(TestVlb):

    def setUp(self):
        super(TestSecurityPolicy, self).setUp()
        self.create_security_policy()

    def test_01_list_security_policies(self):
        sec_policies = list(self.client.security_policies(
            name=TestSecurityPolicy.security_policy.name))
        self.assertGreaterEqual(len(sec_policies), 0)

    def test_02_get_security_policy(self):
        sec_policy = self.client.get_security_policy(
            TestVlb.security_policy)
        self.assertIsNotNone(sec_policy)

    def test_03_find_security_policy(self):
        sec_policy = self.client.find_security_policy(
            TestVlb.security_policy.name)
        self.assertIsNotNone(sec_policy)

    def test_04_update_security_policy(self):
        new_description = 'changed'
        protocols = ["TLSv1.3"]
        ciphers = ["TLS_AES_128_GCM_SHA256"]
        sec_policy = self.client.update_security_policy(
            TestVlb.security_policy,
            description=new_description,
            protocols=protocols,
            ciphers=ciphers
        )
        self.assertEqual(sec_policy['description'], new_description)
        self.assertEqual(sec_policy['protocols'], protocols)
        self.assertEqual(sec_policy['ciphers'], ciphers)

        # cleanup
        self.client.delete_security_policy(
            TestVlb.security_policy
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.osclient.dws.v1 import cluster
from otcextensions.tests.unit.osclient.dws.v1 import fakes

# from osc_lib.cli import format_columns


_COLUMNS = (
    'action_progress',
    'availability_zone',
    'created_at',
    'endpoints',
    'enterprise_project_id',
    'flavor',
    'flavor_id',
    'floating_ip',
    'guest_agent_version',
    'id',
    'is_logical_cluster_enabled',
    'is_logical_cluster_initialed',
    'is_logical_cluster_mode',
    'maintenance_window',
    'name',
    'network_id',
    'nodes',
    'num_free_nodes',
    'num_nodes',
    'num_recent_events',
    'parameter_group',
    'port',
    'private_domain',
    'private_ip',
    'public_domain',
    'public_endpoints',
    'router_id',
    'security_group_id',
    'spec_version',
    'status',
    'sub_status',
    'tags',
    'task_status',
    'updated_at',
    'user_name',
    'version',
)


class TestListClusters(fakes.TestDws):
    objects = fakes.FakeCluster.create_multiple(3)

    column_list_headers = (
        'ID',
        'Name',
        'Num Nodes',
        'Flavor',
        'Status',
        'Version',
        'Created At',
    )

    columns = (
        'id',
        'name',
        'num_nodes',
        'flavor',
        'status',
        'version',
        'created_at',
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                s.name,
                s.num_nodes,
                s.flavor,
                s.status,
                s.version,
                s.created_at,
            )
        )

    def setUp(self):
        super(TestListClusters, self).setUp()

        self.cmd = cluster.ListClusters(self.app, None)

        self.client.clusters = mock.Mock()
        self.client.api_mock = self.client.clusters

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestCreateCluster(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_one()

    columns = _COLUMNS

    data = fakes.gen_data(_cluster, columns, cluster._formatters)

    default_timeout = 1800

    def setUp(self):
        super(TestCreateCluster, self).setUp()

        self.cmd = cluster.CreateCluster(self.app, None)

        self.client.create_cluster = mock.Mock(return_value=self._cluster)
        self.client.get_cluster = mock.Mock(return_value=self._cluster)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_create(self):
        arglist = [
            'test-dws',
            '--flavor', 'dws-flavor',
            '--router-id', 'router-uuid',
            '--network-id', 'network-uuid',
            '--security-group-id', 'sg-uuid',
            '--num-nodes', '3',
            '--num-cn', '2',
            '--port', '8000',
            '--username', 'dbadmin',
            '--password', 'testtest!',
            '--enterprise-project-id', 'enterprise-uuid',
            '--availability-zone', 'test-az',
            '--floating-ip', 'auto',
            '--wait',
        ]
        verifylist = [
            ('name', 'test-dws'),
            ('flavor', 'dws-flavor'),
            ('router_id', 'router-uuid'),
            ('network_id', 'network-uuid'),
            ('security_group_id', 'sg-uuid'),
            ('num_nodes', 3),
            ('num_cn', 2),
            ('port', 8000),
            ('user_name', 'dbadmin'),
            ('user_pwd', 'testtest!'),
            ('enterprise_project_id', 'enterprise-uuid'),
            ('availability_zone', 'test-az'),
            ('floating_ip', 'auto'),
            ('wait', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'name': 'test-dws',
            'num_nodes': 3,
            'num_cn': 2,
            'flavor': 'dws-flavor',
            'router_id': 'router-uuid',
            'network_id': 'network-uuid',
            'security_group_id': 'sg-uuid',
            'user_name': 'dbadmin',
            'user_pwd': 'testtest!',
            'port': 8000,
            'enterprise_project_id': 'enterprise-uuid',
            'availability_zone': 'test-az',
            'public_ip': {
                'public_bind_type': 'auto_assign',
                'eip_id': ''
            }
        }
        self.client.create_cluster.assert_called_with(**attrs)
        self.client.wait_for_cluster.assert_called_with(
            self._cluster.id, wait=self.default_timeout
        )
        self.client.get_cluster.assert_called_with(self._cluster.id)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestRestartCluster(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_one()

    default_timeout = 300

    def setUp(self):
        super(TestRestartCluster, self).setUp()

        self.cmd = cluster.RestartCluster(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.restart_cluster = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_restart(self):
        arglist = [
            self._cluster.name,
            '--wait',
        ]

        verifylist = [
            ('cluster', self._cluster.name),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._cluster.name, ignore_missing=False
        )
        self.client.restart_cluster.assert_called_with(self._cluster)
        self.client.wait_for_cluster.assert_called_with(
            self._cluster.id, wait=self.default_timeout
        )
        self.assertIsNone(result)


class TestClusterPasswordReset(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_one()

    def setUp(self):
        super(TestClusterPasswordReset, self).setUp()

        self.cmd = cluster.ResetPassword(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.reset_password = mock.Mock(return_value=None)

    def test_password_reset(self):
        arglist = [self._cluster.name, '--password', 'TestPasswordReset']

        verifylist = [
            ('cluster', self._cluster.name),
            ('password', 'TestPasswordReset'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._cluster.name, ignore_missing=False
        )
        self.client.reset_password.assert_called_with(
            self._cluster, 'TestPasswordReset'
        )
        self.assertIsNone(result)


class TestScaleOutCluster(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_one()

    default_timeout = 1800

    def setUp(self):
        super(TestScaleOutCluster, self).setUp()

        self.cmd = cluster.ScaleOutCluster(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.scale_out_cluster = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_scale_out(self):
        arglist = [
            self._cluster.name,
            '--add-nodes',
            '2',
            '--wait',
        ]

        verifylist = [
            ('cluster', self._cluster.name),
            ('add_nodes', 2),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._cluster.name, ignore_missing=False
        )
        self.client.scale_out_cluster.assert_called_with(self._cluster, 2)
        self.client.wait_for_cluster_scale_out.assert_called_with(
            self._cluster.id, wait=self.default_timeout
        )

        self.assertIsNone(result)


class TestShowCluster(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_one()
    columns = _COLUMNS
    data = fakes.gen_data(_cluster, columns, cluster._formatters)

    def setUp(self):
        super(TestShowCluster, self).setUp()

        self.cmd = cluster.ShowCluster(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)
        self.client.get_cluster = mock.Mock(return_value=self._cluster)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            self._cluster.id,
        ]

        verifylist = [
            ('cluster', self._cluster.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._cluster.id, ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_dws_cluster',
        ]

        verifylist = [
            ('cluster', 'unexist_dws_cluster'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_cluster = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_cluster.assert_called_with(
            'unexist_dws_cluster', ignore_missing=False
        )


class TestDeleteCluster(fakes.TestDws):
    _cluster = fakes.FakeCluster.create_multiple(2)

    def setUp(self):
        super(TestDeleteCluster, self).setUp()

        self.client.find_cluster = mock.Mock(return_value=self._cluster[0])
        self.client.delete_cluster = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = cluster.DeleteCluster(self.app, None)

    def test_delete(self):
        arglist = [
            self._cluster[0].name,
        ]

        verifylist = [
            ('cluster', arglist),
            ('keep_last_manual_snapshot', 0),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._cluster[0].name, ignore_missing=False
        )
        self.client.delete_cluster.assert_called_with(self._cluster[0].id, 0)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for dws_cluster in self._cluster:
            arglist.append(dws_cluster.name)

        verifylist = [
            ('cluster', arglist),
            ('keep_last_manual_snapshot', 0),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_results = self._cluster
        self.client.find_cluster = mock.Mock(side_effect=find_mock_results)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        find_calls = []
        delete_calls = []
        for dws_cluster in self._cluster:
            find_calls.append(call(dws_cluster.name, ignore_missing=False))
            delete_calls.append(call(dws_cluster.id, 0))
        self.client.find_cluster.assert_has_calls(find_calls)
        self.client.delete_cluster.assert_has_calls(delete_calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._cluster[0].id,
            'unexist_dws_cluster',
        ]
        verifylist = [
            ('cluster', arglist),
            ('keep_last_manual_snapshot', 0),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_results = [self._cluster[0], exceptions.CommandError]
        self.client.find_cluster = mock.Mock(side_effect=find_mock_results)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 Cluster(s) failed to delete.', str(e))

        self.client.delete_cluster.assert_any_call(self._cluster[0].id, 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock
from openstack.tests.unit import base

from otcextensions.sdk.compute.v2 import server


IDENTIFIER = 'fake_id'
ENDPOINT = 'https://ecs.fake.otc.t-systems.com/v1/my-0346-43be-bc65-project'


class TestServer(base.TestCase):

    def setUp(self):
        super(TestServer, self).setUp()
        self.sot = server.Server(id=IDENTIFIER)
        self.resp = mock.Mock()
        self.resp.body = None
        self.resp.json = mock.Mock(return_value=self.resp.body)
        self.resp.status_code = 200
        self.sess = mock.Mock()
        self.sess.post = mock.Mock(return_value=self.resp)
        self.ecs = mock.Mock()
        self.ecs.post = mock.Mock(return_value=self.resp)
        self.sot._connection = mock.Mock(ecs=self.ecs)
        self.sot._prepare_override_url = mock.Mock(return_value=ENDPOINT)

    def test__get_tag_struct(self):
        self.assertDictEqual(
            {'key': 'k1', 'value': 'v1'},
            self.sot._get_tag_struct('k1=v1')
        )
        self.assertDictEqual(
            {'key': 'k1', 'value': ''},
            self.sot._get_tag_struct('k1')
        )

    def test_add_tag(self):
        # Let the translate pass through, that portion is tested elsewhere
        self.sot._translate_response = lambda arg: arg

        result = self.sot.add_tag(self.sess, 'a=b')

        self.assertIsInstance(result, server.Server)

        url = 'cloudservers/%s/tags/action' % (IDENTIFIER)
        body = {
            "action": "create",
            "tags": [{'key': 'a', 'value': 'b'}]
        }
        self.sess.post.assert_called_with(url, json=body)

    def test_remove_tag(self):
        # Let the translate pass through, that portion is tested elsewhere
        self.sot._translate_response = lambda arg: arg

        result = self.sot.remove_tag(self.sess, 'a=b')

        self.assertIsInstance(result, server.Server)

        url = 'cloudservers/%s/tags/action' % (IDENTIFIER)
        body = {
            "action": "delete",
            "tags": [{'key': 'a', 'value': 'b'}]
        }
        self.sess.post.assert_called_with(url, json=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class EntitySpecSub(resource.Resource):
    #: Properties
    #: Protection group ID
    server_group_id = resource.Body('server_group_id')


class SubJob(resource.Resource):
    #: Properties
    #: Task begin time
    begin_time = resource.Body('begin_time')
    #: Error status code
    code = resource.Body('code')
    #: Task end time
    end_time = resource.Body('end_time')
    #: Additional info on job
    entities = resource.Body('entities', type=EntitySpecSub)
    #: SDRS Error code
    error_code = resource.Body('error_code')
    #: Fail reason
    fail_reason = resource.Body('fail_reason')
    #: Job id
    job_id = resource.Body('job_id')
    #: Job type
    job_type = resource.Body('job_type')
    #: Error message
    message = resource.Body('message')
    #: Job status
    status = resource.Body('status')


class EntitySpec(resource.Resource):
    #: Properties
    #: Protection group ID
    server_group_id = resource.Body('server_group_id')
    #: Sub job information
    sub_jobs = resource.Body('sub_jobs', type=list, list_type=SubJob)


class Job(resource.Resource):
    """SDRS Job Resource"""
    base_path = '/jobs'

    # capabilities
    allow_fetch = True

    #: Properties
    #: Task begin time
    begin_time = resource.Body('begin_time')
    #: Error status code
    code = resource.Body('code')
    #: Task end time
    end_time = resource.Body('end_time')
    #: Additional info on job
    entities = resource.Body('entities', type=EntitySpec)
    #: SDRS Error code
    error_code = resource.Body('error_code')
    #: Fail reason
    fail_reason = resource.Body('fail_reason')
    #: Job id
    job_id = resource.Body('job_id')
    #: Job type
    job_type = resource.Body('job_type')
    #: Error message
    message = resource.Body('message')
    #: Job status
    status = resource.Body('status')



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Add master and client nodes to CSS Cluster
"""

import openstack

openstack.enable_logging(True)
conn = openstack.connect()

cluster_name_or_id = 'a4915eea-7a70-4058-bbf5-c391989c10e3'

attrs = {
    'node_type': 'ess-master',
    'flavor': 'ced8d1a7-eff8-4e30-a3de-cd9578fd518f',
    'node_size': 3,
    'volume_type': 'COMMON',
}

cluster = conn.css.find_cluster(cluster_name_or_id, ignore_missing=False)

conn.css.add_cluster_nodes(cluster, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestAclApiBinding(TestApiG):
    gateway_id = "560de602c9f74969a05ff01d401a53ed"

    def setUp(self):
        super(TestAclApiBinding, self).setUp()

    def test_list_apis_for_acl(self):
        attrs = {
            'acl_id': '1142260d5e654b9590a1d329926bea52'
        }
        found = list(self.client.list_apis_for_acl(
            gateway=TestAclApiBinding.gateway_id, **attrs))
        self.assertGreater(len(found), 0)

    def test_list_api_not_bound_to_acl(self):
        attrs = {
            'acl_id': '1142260d5e654b9590a1d329926bea52'
        }
        found = list(self.client.list_api_not_bound_to_acl(
            gateway=TestAclApiBinding.gateway_id, **attrs))
        self.assertGreater(len(found), 0)

    def test_list_acl_for_api(self):
        attrs = {
            'api_id': '64182cc7e77245ebbae8cf3b8522a540'
        }
        found = list(self.client.list_acl_for_api(
            gateway=TestAclApiBinding.gateway_id, **attrs))
        self.assertGreater(len(found), 0)

    def test_bind_acl_to_api(self):
        attrs = {
            "acl_id": "d5645ed3c454492f8d6aa68ab034c6d3",
            "publish_ids": ["293fe0a8e3f04a1ab151bd0d913900a9"]
        }
        result = list(self.client.bind_acl_to_api(
            gateway=TestAclApiBinding.gateway_id,
            **attrs))
        self.assertGreater(len(result), 0)

    def test_unbind_acl(self):
        self.client.unbind_acl(
            gateway=TestAclApiBinding.gateway_id,
            acl='e37364eb93c44ef093d686383354689b')
        attrs = {
            'api_id': '12259302184a4972ac64277537a6aa20'
        }
        found = list(self.client.list_acl_for_api(
            gateway=TestAclApiBinding.gateway_id, **attrs))
        self.assertEqual(len(found), 0)

    def test_unbind_acls(self):
        attrs = {
            "acl_bindings": ["332c5db1458a477b89b2ea741fec94a3"]
        }
        result = list(self.client.unbind_acls(
            gateway=TestAclApiBinding.gateway_id,
            **attrs
        ))
        self.assertGreater(len(result), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from unittest.mock import call

from osc_lib import exceptions

from otcextensions.osclient.smn.v2 import subscription
from otcextensions.tests.unit.osclient.smn.v2 import fakes


class TestListSubscription(fakes.TestSmn):

    objects = fakes.FakeSubscription.create_multiple(3)
    _topic = fakes.FakeTopic.create_one()

    column_list_headers = (
        'ID',
        'Protocol',
        'Topic URN',
        'Owner',
        'Endpoint',
        'Status'
    )

    columns = (
        'id',
        'protocol',
        'topic_urn',
        'owner',
        'endpoint',
        'status'
    )

    data = []

    for s in objects:
        data.append(
            (s.id,
             s.protocol,
             s.topic_urn,
             s.owner,
             s.endpoint,
             s.status))

    def setUp(self):
        super(TestListSubscription, self).setUp()

        self.cmd = subscription.ListSubscription(self.app, None)

        self.client.get_topic = mock.Mock(return_value=self._topic)
        self.client.subscriptions = mock.Mock()
        self.client.api_mock = self.client.subscriptions

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(None)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--topic', '1',
            '--limit', '2',
            '--offset', '3'
        ]

        verifylist = [
            ('topic', '1'),
            ('limit', 2),
            ('offset', 3),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            self._topic,
            limit=2,
            offset=3,
        )


class TestCreateSubscription(fakes.TestSmn):

    _data = fakes.FakeSubscription.create_one()
    _topic = fakes.FakeTopic.create_one()

    columns = ('endpoint', 'id', 'owner', 'protocol', 'remark', 'status')

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateSubscription, self).setUp()

        self.client.get_topic = mock.Mock(return_value=self._topic)
        self.cmd = subscription.CreateSubscription(self.app, None)

        self.client.create_subscription = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            'test-topic',
            '--endpoint', 'test@otce.com',
            '--protocol', 'email',
            '--remark', 'test subscription',
        ]
        verifylist = [
            ('topic', 'test-topic'),
            ('endpoint', 'test@otce.com'),
            ('protocol', 'email'),
            ('remark', 'test subscription'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_subscription.assert_called_with(
            self._topic,
            endpoint='test@otce.com',
            protocol='email',
            remark='test subscription',
        )
        self.assertEqual(self.columns, columns)


class TestDeleteSubscription(fakes.TestSmn):

    _data = fakes.FakeSubscription.create_multiple(2)

    def setUp(self):
        super(TestDeleteSubscription, self).setUp()

        self.client.delete_subscription = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = subscription.DeleteSubscription(self.app, None)

    def test_delete(self):
        arglist = ['subscription-urn']

        verifylist = [
            ('subscription', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_subscription.assert_called_with(
            'subscription-urn', ignore_missing=False)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for data in self._data:
            arglist.append(data.id)

        verifylist = [
            ('subscription', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for data in self._data:
            calls.append(call(data.id, ignore_missing=False))
        self.client.delete_subscription.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].id,
            'unexist_subscription',
        ]
        verifylist = [
            ('subscription', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [None, exceptions.CommandError]
        self.client.delete_subscription = (
            mock.Mock(side_effect=find_mock_result)
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual(
                '1 of 2 SMN Subscription(s) failed to delete.', str(e))

        self.client.delete_subscription.assert_any_call(
            self._data[0].id, ignore_missing=False)
        self.client.delete_subscription.assert_any_call(
            'unexist_subscription', ignore_missing=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.sdrs.v1 import dr_drill as _dr_drill


EXAMPLE = {
    'id': uuid.uuid4(),
    'name': 'Drill-8bcf',
    'status': 'available',
    'server_group_id': uuid.uuid4(),
    'drill_vpc_id': uuid.uuid4(),
    'created_at': '2022-03-06 19:30:34.91',
    'updated_at': '2022-03-06 19:31:16.354',
    'drill_servers': [
        {
            'protected_instance': uuid.uuid4(),
            'drill_server_id': uuid.uuid4()
        }
    ]
}


class TestDRDrill(base.TestCase):

    def setUp(self):
        super(TestDRDrill, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _dr_drill.DRDrill()

    def test_basic(self):
        sot = _dr_drill.DRDrill()
        self.assertEqual('disaster_recovery_drill', sot.resource_key)
        self.assertEqual('disaster_recovery_drills', sot.resources_key)
        self.assertEqual('/disaster-recovery-drills',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertDictEqual({
            'drill_vpc_id': 'drill_vpc_id',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'name',
            'offset': 'offset',
            'server_group_id': 'server_group_id',
            'status': 'status'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        test_dr_drill = _dr_drill.DRDrill(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['id'],
            test_dr_drill.id)
        self.assertEqual(
            EXAMPLE['name'],
            test_dr_drill.name)
        self.assertEqual(
            EXAMPLE['status'],
            test_dr_drill.status)
        self.assertEqual(
            EXAMPLE['server_group_id'],
            test_dr_drill.server_group_id)
        self.assertEqual(
            EXAMPLE['drill_vpc_id'],
            test_dr_drill.drill_vpc_id)
        self.assertEqual(
            EXAMPLE['created_at'],
            test_dr_drill.created_at)
        self.assertEqual(
            EXAMPLE['updated_at'],
            test_dr_drill.updated_at)
        self.assertEqual(
            EXAMPLE['drill_servers'][0]['protected_instance'],
            test_dr_drill.drill_servers[0].protected_instance)
        self.assertEqual(
            EXAMPLE['drill_servers'][0]['drill_server_id'],
            test_dr_drill.drill_servers[0].drill_server_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.vlb import TestVlb


class TestHealthMonitor(TestVlb):

    def setUp(self):
        super(TestHealthMonitor, self).setUp()
        self.create_network()
        self.create_load_balancer()
        self.create_listener()
        self.create_pool()
        self.create_health_monitor()

    def test_01_list_health_monitors(self):
        hms = list(self.client.health_monitors())
        self.assertGreaterEqual(len(hms), 0)

    def test_02_get_health_monitor(self):
        hm = self.client.get_health_monitor(TestVlb.health_monitor)
        self.assertIsNotNone(hm)

    def test_03_find_health_monitor(self):
        hm = self.client.find_health_monitor(TestVlb.health_monitor.id)
        self.assertIsNotNone(hm)

    def test_04_update_health_monitor(self):
        new_delay = 20
        hm = self.client.update_health_monitor(
            TestVlb.health_monitor,
            delay=new_delay,
        )
        self.assertEqual(hm['delay'], new_delay)

        # cleanup
        self.client.delete_health_monitor(
            TestVlb.health_monitor
        )
        self.client.delete_pool(
            TestVlb.pool
        )
        self.client.delete_listener(
            TestVlb.listener
        )
        self.client.delete_load_balancer(
            TestVlb.load_balancer
        )
        self.net_client.delete_ip(
            TestVlb.load_balancer.floating_ips[0]['publicip_id']
        )
        self.addCleanup(self.destroy_network, TestVlb.network)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock
from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.elb.v2 import elb_certificate

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "id": FAKE_ID,
    "name": "fake_name",
    "private_key": "fake",
    "certificate": "fake",
    "update_time": 1499817600,
    "create_time": 1499817600
}


class TestElbCertificate(base.TestCase):

    def setUp(self):
        super(TestElbCertificate, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

    def test_basic(self):
        sot = elb_certificate.Certificate()

        self.assertEqual('/lbaas/certificates', sot.base_path)
        self.assertEqual('certificates', sot.resources_key)
        self.assertIsNone(sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make(self):
        sot = elb_certificate.Certificate(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['update_time'], sot.update_time)
        self.assertEqual(EXAMPLE['create_time'], sot.create_time)

        self.assertDictEqual({
            'limit': 'limit',
            'marker': 'marker'},
            sot._query_mapping._mapping
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from otcextensions.tests.functional.sdk.swr import TestSwr


class TestDomain(TestSwr):

    def setUp(self):
        super(TestDomain, self).setUp()

        self.org_name = "sdk-swr-org-" + uuid.uuid4().hex
        self.repo_name = "sdk-swr-repo-" + uuid.uuid4().hex
        self.org = self.client.create_organization(
            namespace=self.org_name
        )
        self.repo = self.client.create_repository(
            namespace=self.org_name,
            repository=self.repo_name,
            category='linux',
            description='this is a acc test repository',
            is_public=False,
        )

    def tearDown(self):
        super(TestDomain, self).tearDown()
        self.conn.swr.delete_repository(
            self.org_name,
            self.repo_name
        )
        self.conn.swr.delete_organization(self.org_name)

    def test_list_domains(self):
        domains = list(self.client.domains(
            namespace=self.org.namespace,
            repository=self.repo_name
        ))
        self.assertGreaterEqual(len(domains), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os

from openstack import resource
from openstack import exceptions


class CronConfig(resource.Resource):
    name = resource.Body('name', type=str)
    cron = resource.Body('cron', type=str)
    count = resource.Body('count', type=int)
    start_time = resource.Body('start_time', type=int)
    expired_time = resource.Body('expired_time', type=int)


class TacticsConfig(resource.Resource):
    cron_configs = resource.Body(
        'cron_configs', type=list, list_type=CronConfig
    )


class ReservedInstance(resource.Resource):
    base_path = '/fgs/functions/%(function_urn)s/reservedinstances'
    requires_id = False
    _query_mapping = resource.QueryParameters(
        'marker', 'limit', 'urn'
    )

    # Capabilities
    allow_fetch = True
    allow_list = True
    allow_commit = True

    # Properties
    function_urn = resource.URI('function_urn', type=str)
    #: Number of reserved instances.
    count = resource.Body('count', type=int)
    #: Whether to enable the idle mode.
    idle_mode = resource.Body('idle_mode', type=bool)
    tactics_config = resource.Body('tactics_config', type=dict)

    # Attributes
    qualifier_type = resource.Body('qualifier_type', type=str)
    qualifier_name = resource.Body('qualifier_name', type=str)
    min_count = resource.Body('min_count', type=int)
    func_urn = resource.Body('func_urn', type=str)

    @classmethod
    def list(
            cls,
            session,
            paginated=True,
            base_path=None,
            allow_unknown_params=False,
            *,
            microversion=None,
            **params,
    ):
        """This method is a generator which yields resource objects.

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
            a paginated series of responses, or ``False`` if a GET returns only
            one page of data. **When paginated is False only one page of data
            will be returned regardless of the API's support of pagination.**
        :param str base_path: Base part of the URI for listing resources, if
            different from :data:`~openstack.resource.Resource.base_path`.
        :param bool allow_unknown_params: ``True`` to accept, but discard
            unknown query parameters. This allows getting list of 'filters' and
            passing everything known to the server. ``False`` will result in
            validation exception when unknown query parameters are passed.
        :param str microversion: API version to override the negotiated one.
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be sent
            in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the :data:`~openstack.resource.Resource.base_path`
            format string to see if any path fragments need to be filled in by
            the contents of this argument.
            Parameters supported as filters by the server side are passed in
            the API call, remaining parameters are applied as filters to the
            retrieved results.

        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
            :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
            contains invalid params.
        """
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path

        last_part = os.path.basename(base_path)
        if last_part == 'reservedinstanceconfigs':
            cls.resources_key = 'reserved_instances'
        else:
            cls.resources_key = 'reservedinstances'

        cls._query_mapping._validate(params, base_path=base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'},
                params=query_params.copy())
            exceptions.raise_from_response(response)

            data = response.json()

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            for raw_resource in resources:
                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource)
                yield value

            return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.modelartsv1.v1 import job_flavor
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes


class TestListJobFlavors(fakes.TestModelartsv1):
    objects = fakes.FakeJobFlavor.create_multiple(3)

    column_list_headers = (
        "Spec Id",
        "Spec Code",
        "Core",
        "CPU",
        "GPU Num",
        "GPU Type",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.spec_id,
                s.spec_code,
                s.core,
                s.cpu,
                s.gpu_num,
                s.gpu_type,
            )
        )

    def setUp(self):
        super(TestListJobFlavors, self).setUp()

        self.cmd = job_flavor.ListJobFlavors(self.app, None)

        self.client.job_flavors = mock.Mock()
        self.client.api_mock = self.client.job_flavors

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--job-type",
            "inference",
            "--engine-id",
            "1",
            "--project-type",
            "2",
        ]

        verifylist = [
            ("job_type", "inference"),
            ("engine_id", 1),
            ("project_type", 2),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            job_type="inference",
            engine_id=1,
            project_type=2,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class Flavor(resource.Resource):
    base_path = '/flavors'

    resources_key = 'flavors'

    allow_list = True

    # Properties
    #: Availability Zones. *Type: str*
    availability_zones = resource.Body('availableAZ')
    #: Disk capacity range (min,max). *Type: str*
    disk_range = resource.Body('diskrange', type=str)
    #: ID of a flavor. *Type: ID*
    flavor_id = resource.Body('flavor_id', alternate_id=True)
    #: ram. *Type: int*
    ram = resource.Body('ram', type=int)
    #: region. *Type: str*
    region = resource.Body('region')
    #: Instance Type.
    type = resource.Body('type')
    #: cpu count. *Type: int*
    vcpus = resource.Body('cpu', type=int)
    #: Version. *Type: str*
    version = resource.Body('version')

    @classmethod
    def list(cls, session, paginated=True, base_path=None, **params):
        """This method is a generator which yields resource objects.
        This resource object list generator handles pagination and takes query
        params for response filtering.
        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
                               a paginated series of responses, or ``False``
                               if a GET returns only one page of data.
                               **When paginated is False only one
                               page of data will be returned regardless
                               of the API's support of pagination.**
        :param str base_path: Base part of the URI for listing resources, if
                              different from
                              :data:`~openstack.resource.Resource.base_path`.
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be
            sent in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the
            :data:`~openstack.resource.Resource.base_path` format string
            to see if any path fragments need to be filled in by the contents
            of this argument.
        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
                 contains invalid params.
        """
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        cls._query_mapping._validate(params, base_path=base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        # Copy query_params due to weird mock unittest interactions
        response = session.get(
            uri,
            headers={"Accept": "application/json"},
            params=query_params.copy(),
            microversion=microversion,
        )
        exceptions.raise_from_response(response)
        data = response.json()

        if 'versions' in data:
            for ver in data['versions']:
                version = ver['version']
                type = ver['type']
                resources = ver[cls.resources_key]

                if not isinstance(resources, list):
                    resources = [resources]

                for raw_resource in resources:
                    raw_resource.pop("self", None)

                    value = cls.existing(
                        microversion=microversion,
                        connection=session._get_connection(),
                        version=version,
                        type=type,
                        **raw_resource
                    )
                    yield value
        else:
            return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import acl_api_binding


EXAMPLE_ACL_API_BINDING = {
    'gateway_id': 'gateway-67890',
    'id': 'binding-001',
    'api_id': 'api-123',
    'env_id': 'env-1',
    'acl_id': 'acl-456',
    'create_time': '2025-04-01T10:00:00Z'
}

EXAMPLE_ACL_BINDING_FAILURE = {
    'bind_id': 'bind-999',
    'error_code': '403',
    'error_msg': 'Forbidden',
    'api_id': 'api-123',
    'api_name': 'Test API'
}

EXAMPLE_API_FOR_ACL = {
    'gateway_id': 'gateway-67890',
    'api_id': 'api-123',
    'api_name': 'Bound API',
    'api_type': 'REST',
    'api_remark': 'Already bound',
    'env_id': 'env-1',
    'env_name': 'prod',
    'bind_id': 'bind-123',
    'group_name': 'groupA',
    'bind_time': '2025-04-01T09:00:00Z',
    'publish_id': 'pub-321',
    'req_method': 'GET'
}

EXAMPLE_UNBIND_API_FOR_ACL = {
    'gateway_id': 'gateway-67890',
    'id': 'api-999',
    'name': 'Unbound API',
    'group_id': 'group-222',
    'group_name': 'Group B',
    'type': 1,
    'remark': 'Not bound',
    'run_env_name': 'dev',
    'run_env_id': 'env-2',
    'publish_id': 'pub-456',
    'acl_name': 'Test ACL',
    'req_uri': '/test',
    'auth_type': 'NONE',
    'req_method': 'POST'
}

EXAMPLE_ACL_FOR_API = {
    'gateway_id': 'gateway-67890',
    'acl_id': 'acl-456',
    'acl_name': 'Test ACL',
    'entity_type': 'APP',
    'acl_type': 'IP',
    'acl_value': '192.168.1.1',
    'env_id': 'env-1',
    'env_name': 'prod',
    'bind_id': 'bind-001',
    'bind_time': '2025-04-01T08:30:00Z'
}


class TestAclApiBinding(base.TestCase):

    def test_basic(self):
        sot = acl_api_binding.AclApiBinding()
        self.assertEqual('apigw/instances/%(gateway_id)s/acl-bindings',
                         sot.base_path)
        self.assertEqual('acl_bindings', sot.resources_key)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = acl_api_binding.AclApiBinding(**EXAMPLE_ACL_API_BINDING)
        self.assertEqual(EXAMPLE_ACL_API_BINDING['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_ACL_API_BINDING['id'], sot.id)
        self.assertEqual(EXAMPLE_ACL_API_BINDING['api_id'], sot.api_id)
        self.assertEqual(EXAMPLE_ACL_API_BINDING['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_ACL_API_BINDING['acl_id'], sot.acl_id)
        self.assertEqual(EXAMPLE_ACL_API_BINDING['create_time'],
                         sot.create_time)


class TestAclBindingFailure(base.TestCase):

    def test_basic(self):
        sot = acl_api_binding.AclBindingFailure()
        self.assertEqual('failure', sot.resources_key)

    def test_make_it(self):
        sot = acl_api_binding.AclBindingFailure(**EXAMPLE_ACL_BINDING_FAILURE)
        self.assertEqual(EXAMPLE_ACL_BINDING_FAILURE['bind_id'], sot.bind_id)
        self.assertEqual(EXAMPLE_ACL_BINDING_FAILURE['error_code'],
                         sot.error_code)
        self.assertEqual(EXAMPLE_ACL_BINDING_FAILURE['error_msg'],
                         sot.error_msg)
        self.assertEqual(EXAMPLE_ACL_BINDING_FAILURE['api_id'], sot.api_id)
        self.assertEqual(EXAMPLE_ACL_BINDING_FAILURE['api_name'],
                         sot.api_name)


class TestApiForAcl(base.TestCase):

    def test_basic(self):
        sot = acl_api_binding.ApiForAcl()
        self.assertEqual(
            'apigw/instances/%(gateway_id)s/acl-bindings/binded-apis',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('apis', sot.resources_key)

    def test_make_it(self):
        sot = acl_api_binding.ApiForAcl(**EXAMPLE_API_FOR_ACL)
        self.assertEqual(EXAMPLE_API_FOR_ACL['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_API_FOR_ACL['api_id'], sot.api_id)
        self.assertEqual(EXAMPLE_API_FOR_ACL['api_name'], sot.api_name)
        self.assertEqual(EXAMPLE_API_FOR_ACL['api_type'], sot.api_type)
        self.assertEqual(EXAMPLE_API_FOR_ACL['api_remark'], sot.api_remark)
        self.assertEqual(EXAMPLE_API_FOR_ACL['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_API_FOR_ACL['env_name'], sot.env_name)
        self.assertEqual(EXAMPLE_API_FOR_ACL['bind_id'], sot.bind_id)
        self.assertEqual(EXAMPLE_API_FOR_ACL['group_name'], sot.group_name)
        self.assertEqual(EXAMPLE_API_FOR_ACL['bind_time'], sot.bind_time)
        self.assertEqual(EXAMPLE_API_FOR_ACL['publish_id'], sot.publish_id)
        self.assertEqual(EXAMPLE_API_FOR_ACL['req_method'], sot.req_method)


class TestUnbindApiForAcl(base.TestCase):

    def test_basic(self):
        sot = acl_api_binding.UnbindApiForAcl()
        self.assertEqual(
            'apigw/instances/%(gateway_id)s/acl-bindings/unbinded-apis',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('apis', sot.resources_key)

    def test_make_it(self):
        sot = acl_api_binding.UnbindApiForAcl(**EXAMPLE_UNBIND_API_FOR_ACL)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['gateway_id'],
                         sot.gateway_id)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['id'], sot.id)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['name'], sot.name)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['group_id'], sot.group_id)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['group_name'],
                         sot.group_name)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['type'], sot.type)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['remark'], sot.remark)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['run_env_name'],
                         sot.run_env_name)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['run_env_id'],
                         sot.run_env_id)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['publish_id'],
                         sot.publish_id)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['acl_name'], sot.acl_name)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['req_uri'], sot.req_uri)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['auth_type'],
                         sot.auth_type)
        self.assertEqual(EXAMPLE_UNBIND_API_FOR_ACL['req_method'],
                         sot.req_method)


class TestAclForApi(base.TestCase):

    def test_basic(self):
        sot = acl_api_binding.AclForApi()
        self.assertEqual(
            'apigw/instances/%(gateway_id)s/acl-bindings/binded-acls',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('acls', sot.resources_key)

    def test_make_it(self):
        sot = acl_api_binding.AclForApi(**EXAMPLE_ACL_FOR_API)
        self.assertEqual(EXAMPLE_ACL_FOR_API['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_ACL_FOR_API['acl_id'], sot.acl_id)
        self.assertEqual(EXAMPLE_ACL_FOR_API['acl_name'], sot.acl_name)
        self.assertEqual(EXAMPLE_ACL_FOR_API['entity_type'], sot.entity_type)
        self.assertEqual(EXAMPLE_ACL_FOR_API['acl_type'], sot.acl_type)
        self.assertEqual(EXAMPLE_ACL_FOR_API['acl_value'], sot.acl_value)
        self.assertEqual(EXAMPLE_ACL_FOR_API['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_ACL_FOR_API['env_name'], sot.env_name)
        self.assertEqual(EXAMPLE_ACL_FOR_API['bind_id'], sot.bind_id)
        self.assertEqual(EXAMPLE_ACL_FOR_API['bind_time'], sot.bind_time)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class Tag(resource.Resource):
    base_path = '/clusters/%(cluster_id)s/tags'

    # Properties
    cluster_id = resource.URI('cluster_id')
    resources_key = 'tags'
    resource_key = 'tag'

    # Capabilities
    allow_list = True
    allow_create = True
    allow_delete = True

    _query_mapping = resource.QueryParameters('key')

    # Properties
    key = resource.Body('key')
    value = resource.Body('value')

    def manage_tags_batch(self, session, cluster_id, tags, action):
        """
        Manage tags in batch for a cluster (create or delete).
        """
        self.last_tags_sent = tags
        uri = utils.urljoin('clusters', cluster_id, 'tags/action')
        body = {'action': action, 'tags': tags}
        response = session.post(uri, json=body)
        return self._translate_response(response)

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """
        Translates the server response into Tag objects.
        """
        if has_body is None:
            has_body = self.has_body
        exceptions.raise_from_response(response, error_message=error_message)
        tags_list = []
        if has_body and response.status_code == 204:
            if hasattr(self, 'last_tags_sent'):
                tags_list = [
                    Tag.existing(**tag_data)
                    for tag_data in self.last_tags_sent
                ]
        return tags_list



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from openstack import exceptions
from openstack import _log
from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class BaseDCTest(base.BaseFunctionalTest):

    UUID = uuid.uuid4().hex[:10]
    NETWORK_NAME = "test-dc-network-" + UUID
    SUBNET_NAME = "test-dc-subnet-" + UUID
    ROUTER_NAME = "test-dc-router-" + UUID
    IP_VERSION = 4
    CIDR = "192.168.0.0/16"

    def _create_network(self):
        return self.conn.network.create_network(
            name=self.NETWORK_NAME
        )

    def _delete_network(self, network):
        return self.conn.network.delete_network(
            network=network
        )

    def _create_subnet(self, network_id):
        return self.conn.network.create_subnet(
            name=self.SUBNET_NAME,
            network_id=network_id,
            ip_version=self.IP_VERSION,
            cidr=self.CIDR
        )

    def _delete_subnet(self, subnet):
        return self.conn.network.delete_subnet(
            subnet=subnet
        )

    def _create_router(self, subnet_id):
        router = self.conn.network.create_router(
            name=self.ROUTER_NAME
        )
        self.conn.network.add_interface_to_router(
            router=router,
            subnet_id=subnet_id
        )
        return router

    def _delete_router(self, router, subnet_id):
        self.conn.network.remove_interface_from_router(
            router=router,
            subnet_id=subnet_id
        )
        return self.conn.network.delete_router(
            router=router
        )

    def create_test_infra(self):
        network = self._create_network()
        subnet = self._create_subnet(network.id)
        router = self._create_router(subnet.id)
        return {
            "network_id": network.id,
            "subnet_id": subnet.id,
            "router_id": router.id
        }

    def delete_test_infra(self, infra: dict):
        router = self.conn.network.get_router(infra.get("router_id"))
        subnet = self.conn.network.get_subnet(infra.get("subnet_id"))
        network = self.conn.network.get_network(infra.get("network_id"))
        if router:
            self._delete_router(router, subnet.id)
        if subnet:
            self._delete_subnet(subnet)
        if network:
            self._delete_network(network)

    def setUp(self):
        super(BaseDCTest, self).setUp()
        self.infra = self.create_test_infra()

    def tearDown(self):
        try:
            self.delete_test_infra(self.infra)
        except exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(BaseDCTest, self).tearDown()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class NodeSpec(resource.Resource):
    #: Number of available nodes.
    available_count = resource.Body("available_count", type=int)
    #: Number of nodes.
    count = resource.Body("count", type=int)
    #: Node specifications.
    specification = resource.Body("specification")


class ServiceCluster(resource.Resource):
    base_path = "/clusters"

    resources_key = "clusters"

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        "project_id",
        "name",
        "status",
        "marker",
        "limit",
        "sort_by",
        "order",
        cluster_name="name",
        offset="marker",
    )

    # Properties
    #: Number of available CPU cores.
    allocatable_cpu_cores = resource.Body("allocatable_cpu_cores", type=float)
    #: Number of available GPU cores.
    allocatable_gpus = resource.Body("allocatable_gpus", type=float)
    #: Number of available memory resources.
    allocatable_memory = resource.Body("allocatable_memory", type=int)
    #: Charging Mode.
    charging_mode = resource.Body("charging_mode")
    #: Cluster ID.
    cluster_id = resource.Body("cluster_id")
    #: Cluster name.
    cluster_name = resource.Body("cluster_name")
    #: Time when a cluster is created, in milliseconds calculated
    #:  from 1970.1.1 0:0:0 UTC.
    created_at = resource.Body("created_at", type=int)
    #: Cluster description.
    description = resource.Body("description")
    #: Max Node Count.
    max_node_count = resource.Body("max_node_count", type=int)
    #: Node configuration.
    nodes = resource.Body("nodes", type=NodeSpec)
    #: Order ID.
    order_id = resource.Body("order_id")
    #: User to which a cluster belongs.
    owner = resource.Body("owner")
    #: Number of subscription periods.
    period_num = resource.Body("period_num", type=int)
    #: Subscription period type.
    period_type = resource.Body("period_type")
    #: Product ID.
    product_id = resource.Body("product_id")
    #: Project to which a cluster belongs.
    project = resource.Body("project")
    #: Services Count
    services_count = resource.Body("services_count", type=dict)
    #: Cluster status. The value can be
    #:  `deploying`, `running`, `concerning`, or `abnormal`.
    status = resource.Body("status")
    #: Tenant to which a cluster belongs.
    tenant = resource.Body("tenant")
